
/* 
 * SQARView.m -- Implementation file for the SQARView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "SQARView.h"
#import "drawingFuncs.h"
#import "nxlcau32.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 33				/* maximum binomial coefficient */
#define NG 243				/* number of Golay Classes	*/

extern char ascrule[KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char sqrule[];			/* evolutionary rule */

extern char    arr1[], arr2[];
       int             brr2[AL];
extern double  bp[];			/* Bernstein coefficients  */
extern int     tus[TS];			/* totalistic use stats    */
extern int     lf[];			/* number of live cells	   */
       double  flf[KK];			/* number of live cells    */

extern NXColor colst[KK];    		/* Array for state colors    */

int    checkBoard=NO;
int    drawARBack=YES;
int    drawBnstein=NO;
int    drawEMeanPH=NO;
int    drawMeanPH=NO;
int    drawMosaic=NO;
int    drawFreqs=NO;
int    drawPlane=NO;
int    drawSmall=NO;
int    drawTrajec=NO;
int    slowFreqs=NO;
int    smallSmall=NO;
int    smallMosaic=NO;

int    lastFreq=0;			/* 0 = none, 1 = traj, 2 = return */
int    lastPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lastReturn=0;			/* 0 = none, 1 = alge, 2 = trajec */
int    moTiles=3;
int    pGrid=NO;
int    pDots=NO;
int    trajLen=10;
int    repPlEv;
int    repEMFg;
int    rturnNum=1;

int    gtb[42];

extern int    drawview;
extern int    co, dn;

void    argfram(int n, int l);
void    argdiag(int n, int l);
void    aronegen(char *aa, int *bb, int m, int n); 
void    asctobin();
void	asctoqar();
void	asczero();
void    bargraf(int m, int n);
void    bdot(int i, int j, int l);
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2, double p3);
void    ranpatch(double p[KK], int l, int m);
void    renewarry();
void    savearry();
void    screen(char *aa, int m, int n);
void    statecount(char *aa, int m, int n);
void    suggrule(int i);
void    videodot(int i, int j, int l);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK]);
void    zerline();

@implementation SQARView
 
/* -------------------------------- S Q A R -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadSqar:sender {
  if (sqarWindow==nil) {
    if (![NXApp loadNibSection: "sqar.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=2;
  [SqarWindow makeKeyAndOrderFront: sender];
  [self copyRule: sender];
return self;
}

void drawsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void drawcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void manysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printSQAR:sender {
   if (keyWindow==SqarWindow) 
	{[SqarWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lastPlane) {
      case 0: break;
      case 1: drawPlane=YES;   break;
      case 2: drawMosaic=YES;  break;
      case 3: drawSmall=YES;   break;
      case 4: smallMosaic=YES; break;
      default: break;}
    [keyWindow smartPrintPSCode: self];
    switch (lastPlane) {
      case 0: break;
      case 1: drawPlane=NO;   break;
      case 2: drawMosaic=NO;  break;
      case 3: drawSmall=NO;   break;
      case 4: smallMosaic=NO; break;
      default: break;}
    }
   else if (keyWindow==rtrnWindow) {
    switch (lastReturn) {
      case 0: break;
      case 1: drawMeanPH=YES;  break;
      case 2: drawTrajec=YES;  break;
      case 3: drawEMeanPH=YES; break;
      default: break;}
    [keyWindow smartPrintPSCode: self];
    switch (lastReturn) {
      case 0: break;
      case 1: drawMeanPH=NO;  break;
      case 2: drawTrajec=NO;  break;
      case 3: drawEMeanPH=NO; break;
      default: break;}
    }
   else if (keyWindow==freqWindow) {
    switch (lastFreq) {
      case 0: break;
      case 1: slowFreqs=YES; break;
      case 2: drawFreqs=YES; break;
      default: break;}
    [keyWindow smartPrintPSCode: self];
    switch (lastFreq) {
      case 0: break;
      case 1: slowFreqs=NO; break;
      case 2: drawFreqs=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQAR:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQAR:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copySqarView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [SqarView getBounds: &zbounds];
  [SqarView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [freqView getBounds: &zbounds];
  [freqView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copyThatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=9;
  drawARBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [rtrnView getBounds: &zbounds];
  [rtrnView writePSCodeInside: &zbounds to: pBoard]; 
  drawARBack=YES;
  *flag = NO;
return self;
}

- copySQAR:sender {
   savearry();
   if (keyWindow==SqarWindow) {[self copySqarView: self];}
   else if (keyWindow==freqWindow) switch (lastFreq) {
          case 0: break;
          case 1: [self copyWhoosit: &slowFreqs]; break;
          case 2: [self copyWhoosit: &drawFreqs]; break;
          default: break;}
   else if (keyWindow==planeWindow) switch (lastPlane) {
        case 0: break;
        case 1: [self copyWhatsit: &drawPlane];   break;
        case 2: [self copyWhatsit: &drawMosaic];  break;
        case 3: drawview=7;  [self copyWhatsit: &drawSmall];   break;
        case 4: drawview=11; [self copyWhatsit: &smallMosaic]; break;
        default: break;}
   else if (keyWindow==rtrnWindow) switch (lastReturn) {
        case 0: break;
        case 1: [self copyThatsit: &drawMeanPH];  break;
        case 2: [self copyThatsit: &drawTrajec];  break;
        case 3: [self copyThatsit: &drawEMeanPH]; break;
        default: break;}
   renewarry();
return self;
}

- drawPlEvol:sender {int i;
  repPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawPlane=YES;
  asctobin();
  if (repPlEv==0) [self display]; else for (i=0; i<repPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: repPlEv];
  drawPlane=NO;
  lastPlane=1;
return self;
}

- printPlEvol:sender {
    repPlEv=0;
    drawview=7;
    drawPlane=YES;
    [keyWindow smartPrintPSCode: self];
    drawPlane=NO;
return self;
}

- drawMosaic:sender {
    drawview=11;
    drawMosaic=YES;
    [self display];
    drawMosaic=NO;
    lastPlane=2;
return self;
}

- printMosaic:sender {
    drawview=11;
    drawMosaic=YES;
    [keyWindow smartPrintPSCode: self];
    drawMosaic=NO;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1],p[2]);
return self;
}

- sCellLine:sender {
  smallSmall=YES;
  [planeView display];
  smallSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.333,0.333,0.333); return self;}

void ranpatch(p,l,m) double p[KK]; int l, m; {int a, b, c, i, j, k;
  for (i=0; i<l; i++) for (j=0; j<m; j++) {
    k=m*i+j;
    a=(int)(1000.0*p[0]);
    b=a+(int)(1000.0*p[1]);
    c=random()%1000;
    if (c<a) arr1[k]=0; else if (c<b) arr1[k]=1; else arr1[k]=2;
    }
  }

- checkerBoard:sender {
  checkBoard=YES;
  [planeView display];
  checkBoard=NO;
return self;
}

- uprule:sender {suggrule(0); [rulePanel setAllRules: self]; return self;}
- lerule:sender {suggrule(1); [rulePanel setAllRules: self]; return self;}
- dorule:sender {suggrule(3); [rulePanel setAllRules: self]; return self;}
- rirule:sender {suggrule(2); [rulePanel setAllRules: self]; return self;}
- idrule:sender {suggrule(4); [rulePanel setAllRules: self]; return self;}
- mjrule:sender {suggrule(5); [rulePanel setAllRules: self]; return self;}
- earule:sender {suggrule(6); [rulePanel setAllRules: self]; return self;}
- serule:sender {suggrule(7); [rulePanel setAllRules: self]; return self;}

void suggrule(i) int i; {int j, k[KK], i0, i1, i2, i3, i4;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: for (j=0; j<KK; j++) k[j]=0;
	      k[i0]++; k[i1]++; k[i2]++; k[i3]++; k[i4]++;
	      if      (k[0]>2) j=0;
	      else if (k[1]>2) j=1;
	      else if (k[2]>2) j=2; 
	      else if (k[0]==1) j=0; 
	      else if (k[1]==1) j=1; 
	      else if (k[2]==1) j=2;
	         else           j=i4; 
	      break;
      case 6: j=(i4+1)%KK;
	      if (j==i0 || j==i1 || j==i2 || j==i3) break;
	      else j=i4; break;
      case 7: j=(i4+1)%KK;
	      if (j==i0 || j==i1 || j==i2 || j==i3) break;
      	      j=(KK+i4-1)%KK;
	      if (j==i0 || j==i1 || j==i2 || j==i3) break;
	      else j=i4; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawSmall=YES;
    asctoqar();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&repPlEv);
    [self display];
    if (repPlEv>0) for (k=0; k<repPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawSmall=NO;
    lastPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawSmall=YES;
    repPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawSmall=NO;
return self;
}

- drawSmMosaic:sender {
    moTiles=[moTies selectedRow]+2;
    drawview=11;
    smallMosaic=YES;
    asctoqar();
    savearry();
    [self display];
    renewarry();
    smallMosaic=NO;
    lastPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajLen);
    pDots=[probDots state];
    pGrid=[probGrid state];
    drawview=9;
    drawTrajec=YES;
    asctoqar();
    savearry();
    [self display];
    renewarry();
    drawTrajec=NO;
    lastReturn=2;
return self;
}

- printTraj:sender {
    drawview=9;
    drawTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawTrajec=NO;
return self;
}

- drawFreq:sender {
    drawview=13;
    drawFreqs=YES;
    asctoqar();
    savearry();
    [self display];
    renewarry();
    drawFreqs=NO;
    lastFreq=2;
return self;
}

- printFreq:sender {
    drawview=13;
    drawFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawFreqs=NO;
return self;
}

- slowFreq:sender {
    drawview=13;
    slowFreqs=YES;
    asctoqar();
    savearry();
    [self display];
    renewarry();
    slowFreqs=NO;
    lastFreq=1;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowFreqs=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (m,n) */
void manyrect(i,j,u,v,m,n) int i, j, u, v, m, n; {int k, l;
  asctoqar();
  for (k=0; k<m; k++) for (l=0; l<n; l++) videodot(i*m+k,j*n+l,arr1[k*n+l]);
  aronegen(arr1,brr2,m,n);
}

/* m = high/moTiles, n = wide/moTiles */
void smallgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)moTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<moTiles+1; i++) {
  PSmoveto((double)(en*i),0.0); PSlineto((double)(en*i),mo*em);}
  for (i=0; i<moTiles+1; i++) {
  PSmoveto(0.0,(double)(em*i)); PSlineto(mo*en,(double)(em*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=9;
    drawMeanPH=YES;
    [self display];
    drawMeanPH=NO;
    [window setDocEdited:YES];
    lastReturn=1;
return self;
}

- drawEmpBern:sender {
    sscanf([[sender selectedCell] title],"%d",&repEMFg);
    drawview=9;
    drawEMeanPH=YES;
    savearry();
    [self display];
    renewarry();
    drawEMeanPH=NO;
    [window setDocEdited:YES];
    lastReturn=3;
return self;
}

- printRule:sender {
   drawview=12;
   drawBnstein=YES;
   [keyWindow printPSCode:self];
   drawBnstein=NO;
return self;
}

- readWRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

      asczero(ascrule);
//	if ([[theZMatrix cellAt: 3: 0] state]) ascrule[0][0][0][0][0]='1';
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqarWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theTMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {char ch[WNML];
//	[[theZMatrix cellAt: 3: 0] setIntValue: ascrule[0][0][0][0][0]-'0'];
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqarWindow setDocEdited: NO];
//     [bernView drawBern: nil];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new SQARView object.  First, an initFrame: message is sent 
 * to super to initialize SQARView as a View.  Next, the SQARView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the SQARView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the SQARView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although SQARView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n;
        double           p[KK], q[KK], r, s, t, mo, en;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawARBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawBnstein==YES) {
	  for (i=0; i<TS; i++) bp[i]=0.0;
	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
		if ([[theZMatrix cellAt: j: i] state]) bp[gtb[i]]+=1.0;
		if ([[theOMatrix cellAt: j: i] state]) bp[gtb[i]+1]+=1.0;
		}
	  argfram(50,3);
	  argdiag(50,3);
	  bargraf(TS-1,50);
	  }

	if (drawTrajec==YES) {
	     PSsetlinewidth(0.1);
	     PSsetgray(NX_BLACK);
	     if (pGrid) linegrid(10); else linegrid(1);
	     PSstroke();
	  statecount(arr1,64,64);
	  for (j=0; j<KK; j++) flf[j]=lf[j];
	  if (pDots) ; else videotrline(flf,0);
	  for (i=0; i<trajLen; i++) {
	    statecount(arr1,64,64);
	    for (j=0; j<KK; j++) flf[j]=lf[j];
	  if (pDots) videotrdot(flf,3); else videotrline(flf,3);
	    aronegen(arr1,brr2,64,64);
	    }
	  PSstroke();
	  }

	if (drawFreqs==YES) {
	  argfram(125,4);
	  for (i=0; i<125; i++) {
	    aronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>33) for (l=0; l<tus[j]; l+=34) bdot(i,k++,j%4);
	    bdot(i,lf[1]/33,4);
	    }
	  }

	if (slowFreqs==YES) {
	  argfram(125,3);
	  for (i=0; i<125; i+=3) {
	    aronegen(arr1,brr2,64,64);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>33) for (l=0; l<tus[j]; l+=34) {
		  bdot(i,k,j%4);
		  bdot(i+1,k,j%4);
		  bdot(i+2,k,j%4);
		  k++;
		  }
	    bdot(i,(125*lf[1])/4096,4);
	    bdot(i+1,(125*lf[1])/4096,4);
	    bdot(i+2,(125*lf[1])/4096,4);
	    }
	  }

        if (checkBoard==YES) {
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);
	  i=(int)(plotRect.size.height/4.0);
	  zerline();
        for (k=0; k<(i/3); k++) {
	   for (l=0; l<(j/3); l++)     arr1[k*j+l]=0;
           for (l=j/3; l<2*(j/3); l++) arr1[k*j+l]=1;
           for (l=2*(j/3); l<j; l++)   arr1[k*j+l]=2;
           }
        for (k=i/3; k<2*(i/3); k++) {
	   for (l=0; l<(j/3); l++)     arr1[k*j+l]=1;
           for (l=j/3; l<2*(j/3); l++) arr1[k*j+l]=2;
           for (l=2*(j/3); l<j; l++)   arr1[k*j+l]=0;
           }
        for (k=2*(i/3); k<i; k++) {
	   for (l=0; l<(j/3); l++)     arr1[k*j+l]=2;
           for (l=j/3; l<2*(j/3); l++) arr1[k*j+l]=0;
           for (l=2*(j/3); l<j; l++)   arr1[k*j+l]=1;
           }
	  }

        if (smallSmall==YES) {
	  mo=(double)moTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);           /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  screen(arr1,i,j);
	  if(repPlEv>0) aronegen(arr1,brr2,i,j);
	  }

        if (smallMosaic==YES) {
	  mo=(double)moTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  for (k=0; k<moTiles; k++) for (l=0; l<moTiles; l++) {
	    manyrect(k,l,moTiles,moTiles,i,j);
	    [window flushWindow];
	    }
	  smallgrid(i,j);
	  }

        if (drawMeanPH==YES) {
	  pdiff(100,6);
	  if (YES) {
	     PSsetlinewidth(0.0);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	     }
	}

        if (drawEMeanPH==YES) {
	     PSsetlinewidth(0.0);
	     PSsetgray(NX_BLACK);
	     linegrid(10);
	     PSstroke();
	n=100;
	en=1.0/((double)(n));
	for (i=0; i<=n; i++) for (j=0; i+j<=n; j++) {
	  p[0]=((double)(i))*en;
	  p[1]=((double)(j))*en;
	  p[2]=1.0-p[0]-p[1];
	  ranpatch(p,32,32);
	  for (l=0; l<repEMFg; l++) aronegen(arr1,brr2,32,32);
	  for (k=0; k<KK; k++) flf[k]=(double)lf[k];
	  for (k=0, s=0.0; k<KK; k++) s+=flf[k];
	  for (k=0; k<KK; k++) q[k]=p[k]-flf[k]/s;
	  for (k=0, t=0.0; k<KK; k++) t+=q[k]*q[k];
	  videotrdot(p,sqco(t,i));
	  }
	}

        if (drawPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) Plane Evolution"];
	drawsquare();
	break;
     case 3: 
	[window setTitle: "3-D Solid Tetrahedron Evolution"];
	drawcube();
	drawgrid();
	break;
     default: [window setTitle: "SQAR31 Plane Evolution"];break;}
	}

        if (drawMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (3,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     default: [window setTitle: "SQAR31 Plane Evolution"];break;}
	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the SQARView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the SQARView.  The SQARView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the SQARView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the SQARView, the SQARView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end SQARView.m */
