
/* lcau3t.c				*/
/* (3,3/2) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1990		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB21.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 20 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN21.C for option 'd' [HVM]		*/
/* 11 March 1989 - x option in trule [HVM]			*/
/* 16 March 1989 - *+- option to change screen color [HVM]	*/
/* 17 September 1989 - directory for demonstration rules [HVM]	*/
/* 14 October 1989 - modified for r = 3/2 [HVM]			*/
/* 10 February 1990 - Garden of Eden option [HVM] 		*/

# include <bdos.h>
# include "lcau3t.h"

# define LROW		 9	/* top row of line panel  */
# define DROWS		20	/* rows in demo menu	  */
# define DCOLS		32	/* second col in demo m.  */

# define NX          	14	/* number of sample rules */

char ixrule[NX][TS]= {

	"000101011",	/*  1 */
	"000101101",	/*  2 */
	"000101101",	/*  3 */
	"000101101",	/*  4 */
	"000101101",	/*  5 */
	"000101111",	/*  6 */
	"002011220",	/*  7 gridwork */
	"000110111",	/*  8 */
	"001011111",	/*  9 */
	"122201202",	/* 10 class iv */
	"010110101",	/* 11 */
	"011010011",	/* 12 */
	"200011202",	/* 13 colored snow */
	"110100101"	/* 14 */

	};

char   ascrule[KK][KK][KK][KK];
char   auxrule[KK][KK][KK][KK];				/* auxiliary transition table */
char   trule[TS];
int    binrule[KK][KK][KK][KK];
int    arr1[AL], arr2[AL];
int    ru, ru0, ru1, ru2, ru3;				/* rule cursor */
int    li, lj;						/* cell pointer */
int    di, dj;						/* demo pointer */
int    sc;						/* screen colors */
int    ix0, iy0;					/* origin for pen moves */

main()
{
int  i, j, i0, i1, i2, i3;
int  more='r';
char a, b, c;

for (i=0; i<TS; i++) trule[i]='0';

ru=6; ru0=0; ru1=0; ru2=0; ru3=0;
li=SL/2; lj=0;						/* line editor cursor */
sc=0;

    videopalette(WHCYMAG);				/* white/cyan/magenta */

    tuto();
    while (!kbdst()) rand();				/* wait for keypress */
    kbdin();						/* ignore it */
    videomode(T80X25);
    videoscroll(0,0,CR,71,0,3);				/* menu on blue background */
    videoscroll(CL+8,0,24,71,0,3);
    i=(rand()%NX);
    di=i%DROWS;
    dj=(i/DROWS==1?DCOLS:1);
    xtoasc(i);
    auxblnk();
    ranlin();						/* random initial array */

    while (more!='n') {					/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {					/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {
        case '!': sc=7; break;
	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,CM,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin();
	      evolve();
	      }
            videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	      ascrule[i0][i1][i2][i3]=trule[i0+i1+i2+i3];
	    videocursor(0,CM,0);
	    rmenu();
	    break;
	case 't':					/* probability calculation */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,CM,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline(8,40);
	    videocursor(0,CM,0);
	    lmenu();
	    break;
        case '#':					/* read stored rule */
	    xmenu(NX);
	    xtoasc(lim(1,dmenu(),NX)-1);
	    rmenu();
            break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(i);
	      ranlin();
	      evolve();
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
	case 'e':					/* Garden of Eden */
	    edard();
	    rmenu();
	    lmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'w':					/* Wolfram rule # */
	    i=numin(0);
	    wmenu(i);
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	      {ascrule[i0][i1][i2][i3]='0'+i%KK; i/=KK;}
	    rmenu();
	    break;
	case 'X':					/* random rule */
	    xblnk();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	      if ((KK*(KK*i0+i1)+i2)%4 == 0) i=rand();
	      ascrule[i0][i1][i2][i3]='0'+i%KK;
	      i/=KK;
	    }}}}
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    i=rand();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	      if (i == 0) i=rand();
	      if (auxrule[i0][i1][i2][i3]==' ') {
		ascrule[i0][i1][i2][i3]='0'+i%KK;
		i/=KK;
		}
	      }}}}
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin();
            lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	    ascrule[i3][i2][i1][i0]=ascrule[i0][i1][i2][i3];      
	    }}}}
	    break;
	case 'B':					/* begin barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[0][a-'0'][b-'0'][a-'0']=a;
	    ascrule[1][a-'0'][b-'0'][a-'0']=a;
	    ascrule[2][a-'0'][b-'0'][a-'0']=a;
	    rmenu();
	    break;
	case 'E':					/* end barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][0]=b;
	    ascrule[a-'0'][a-'0'][a-'0'][1]=b;
	    ascrule[a-'0'][a-'0'][a-'0'][2]=b;
	    rmenu();
	    break;
	case 'L':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0']=c;
	    rmenu();
	    break;
	case 'R':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0']=a;
	    rmenu();
	    break;
	case 'S':					/* still life link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0']=b;
	    rmenu();
	    break;
	case '.':					/* one cycle of evolution */
	    asctobin();
	    onegen(AL);
	    lmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<40; j++) arr1[40*i+j]=arr1[j];}
	    lmenu();
	    break;
	case '~':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve();
    videocursor(0,0,0);
    scrstr("?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}    

/* list of demonstration rules */
int dmenu() {int ii, jj; char c;
  videomode(T80X25);
  videocursor(0,0,0);  printf(" 1 - ");
  videocursor(0,1,0);  printf(" 2 - ");
  videocursor(0,2,0);  printf(" 3 - ");
  videocursor(0,3,0);  printf(" 4 - ");
  videocursor(0,4,0);  printf(" 5 - ");
  videocursor(0,5,0);  printf(" 6 - ");
  videocursor(0,6,0);  printf(" 7 - gridwork ");
  videocursor(0,7,0);  printf(" 8 - ");
  videocursor(0,8,0);  printf(" 9 - ");
  videocursor(0,9,0);  printf("10 - class iv");
  videocursor(0,10,0);  printf("11 - ");
  videocursor(0,11,0);  printf("12 - ");
  videocursor(0,12,0);  printf("13 - colored snow");
  videocursor(0,13,0);  printf("14 - ");
  videocursor(0,14,0);  printf("15");
  videocursor(0,15,0);  printf("16");
  videocursor(0,16,0);  printf("17");
  videocursor(0,17,0);  printf("18");
  videocursor(0,18,0);  printf("19");
  videocursor(0,19,0);  printf("20");
  videocursor(0,0,DCOLS);  printf("21");
  videocursor(0,1,DCOLS);  printf("22");
  videocursor(0,2,DCOLS);  printf("23");
  videocursor(0,3,DCOLS);  printf("24");
  videocursor(0,4,DCOLS);  printf("25");
  videocursor(0,5,DCOLS);  printf("26");
  videocursor(0,6,DCOLS);  printf("27");
  videocursor(0,7,DCOLS);  printf("28");
  videocursor(0,8,DCOLS);  printf("29");
  videocursor(0,9,DCOLS);  printf("30");
  videocursor(0,10,DCOLS);  printf("31");
  videocursor(0,11,DCOLS);  printf("32");
  videocursor(0,12,DCOLS);  printf("33");
  videocursor(0,13,DCOLS);  printf("34");
  videocursor(0,14,DCOLS);  printf("35");
  videocursor(0,15,DCOLS);  printf("36");
  videocursor(0,16,DCOLS);  printf("37");
  videocursor(0,17,DCOLS);  printf("38");
  videocursor(0,18,DCOLS);  printf("39");
  videocursor(0,19,DCOLS);  printf("40");
  videocursor(0,21,0);  printf("move cursor with arrows, select with <cr>\n");
  videocursor(0,22,0);  printf("position at top with <, bottom with >.\n");
  ii=DROWS; jj=DCOLS; c='!'; /* ii = max # rows, jj = second column */
  do {
    videocursor(0,di,dj);
    while (!kbdst()) videocursor(0,di,dj);
    c=kbdin();
      switch (c) {
      case '\013': case '\310': if (di==0) di=ii-1; else di--; break; /* up arrow */
      case '\012': case '\320': if (di==ii-1) di=0; else di++; break; /* down arrow */
      case '\014': case '\315': case ' ': dj=(dj==1?jj:1); break;     /* forward */
      case '\010': case '\313': dj=(dj==1?jj:1); break;		      /* backward */
      case '<': di=0; break;					      /* home */
      case '>': di=ii-1; break;					      /* bottom */
      default: break;
      }
    } while (c!='\015');
  videomode(T80X25);
  return(di+2*(dj==1?0:1)+1);
}

/* edit the rule */
edrule() {char c;
  while (0<1) {
    cscrrul();
    videocursor(0,5,ru);
    c=kbdin();
    if (c=='\015') break;				/* carriage return exits */
    switch (c) {
    case '0': case '1':	case '2':			/* state */
      ascrule[ru0][ru1][ru2][ru3]=c;
      videocattr(0,c,3,1);
      rupl();
      break;
    case '\011':					/* tab = next quad */
      ru3=0; ru2=0; ru1=0; ru0++;
      if (ru0==KK) ru0=0;
      ru=KK*KK*KK*ru0+6;
      break;
    case ' ': case '\315': rupl(); break;				 /* advance */
    case '\010': case '\313': rumi(); break;				 /* backspace */
    case '\013': case '\310': auxrule[ru0][ru1][ru2][ru3]='+'; rupl(); break; /* up arrow */
    case '\012': case '\320': auxrule[ru0][ru1][ru2][ru3]=' '; rupl(); break; /* down arrow */
    case 'z': asczero();
    default: break;
    }
  }
}

/* advance the rule cursor */
rupl() {
  ru3++;
  if (ru3==KK) {ru3=0; ru2++;}
  if (ru2==KK) {ru2=0; ru1++;}
  if (ru1==KK) {ru1=0; ru0++;}
  if (ru0==KK) {ru0=0; }
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* retract the rule cursor */
rumi() {
  if (ru3!=0) ru3--; else {ru3=KK-1;
  if (ru2!=0) ru2--; else {ru2=KK-1;
  if (ru1!=0) ru1--; else {ru1=KK-1;
  if (ru0!=0) ru0--; else {ru0=KK-1;
     }}}}
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* edit totalistic rule */
edtrule() {char c; int  i, k;
  i=0;
  while (i<TS) {
    c=trule[i];
    videocursor(0,3,56+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1': case '2':		/* state */
        trule[i]=c;
        videocattr(0,c,CR,1);
        i++;
        break;
      case ' ': case '\315':			/* space = advance */
        i++;
        break;
      case '\010': case '\313':			/* backspace */
        if (i!=0) i--;
	break;
      case 'x':					/* random selection */
	for (i=0; i<TS; i++) {
	  if (i%8==0) k=rand();
	  trule[i]='0'+k%KK; k/=KK;}
	i=0;
	tscrrul();
	break;
      default: break;
      }
  }
}

/* edit the line of cells */
edline(nr,nc) int nr, nc; {
char c;
int  i, j, k, l, m;
int  aa, bb, cc, dd;
int  ww, xx, yy, zz;
int  ll;

  m=0; /* refresh trigger */
  cscrrul();
  videocursor(0,19,0);
  scrstr("insert states with 0, 1; move cursor");
  videocursor(0,20,0);
  scrstr("with keyboard arrows, space or backspace.");
  videocursor(0,21,0);
  scrstr("carriage return exits");
  while (0<1) {
  ll=lj;
  videocursor(0,lj+LROW,li);
  c=kbdin();
  if (c == '\015') {videoscroll(19,0,24,70,0,3); break;}
  switch (c) {
  case '0': case '1': case '2':				/* enter state */
    arr1[nc*lj+li]=c-'0';
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\012': case '\320':				/* downwards */
    lj++; if (lj>=nr) lj=0;
    break;
  case '\013': case '\310':				/* upwards */
    if (lj>0) lj--; else lj=nr-1;
    break;
  case '\014': case '\315': case ' ':			/* forward */
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\010': case '\313':				/* backward */
    if (li>0) li--;
      else {li=nc-1; if (lj>0) lj--; else lj=nr-1;}
    break;
  case '<':						/* left margin */
    li=0;
    break;
  case '>':						/* right margin */
    li=nc-1;
    break;
  case 'z':						/* clear single row */
    for (k=0; k<nc; k++) arr1[nc*lj+k]=0;
    break;
  case 'Z':						/* clear full array */
    for (k=0; k<AL; k++) arr1[k]=0;
    lmenu();
    break;
  case '^':						/* copy line above */
    for (k=0; k<nc; k++)
      arr1[nc*lj+k]=arr1[nc*(lj-1)+k];
    break;
  case 'x':						/* unmark transitions */
    auxblnk();
    cscrrul();
    break;				
  case 'q':						/* clean up the area */
    ulmenu(3);
    break;
  case '=':		/* insert point transition in ascrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    c=arr1[yy+nc]+'0';
    ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
    rmenu(); 
    cscrrul();
    break;
  case '+':		/* set marker for transition in auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]='+';
    rmenu(); 
    cscrrul();
    break;
  case '-':		/* remove marker for transition from auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=' ';
    rmenu(); 
    cscrrul();
    break;
  case '*':		/* insert transitions for whole line */
    if (lj==0) break;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      c=arr1[yy+nc]+'0';
      ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
      auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]=c;
      }
    rmenu(); 
    cscrrul();
    break;
  case '.':		/* point evolution */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    arr1[yy+nc]=ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]-'0';
    break;
  case '?':		/* evolution of whole line */
    if (lj==nr-1) {for (i=0; i<AL-nc; i++) arr1[i]=arr1[i+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      arr1[yy+nc]=ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]]-'0';
      }
    break;
  case '/':		/* conditional evolution */
    if (lj==nr-1) {for (k=0; k<AL-nc; k++) arr1[k]=arr1[k+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      j=0;
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx]; if (aa>KK-1) j=KK;
      bb=arr1[yy]; if (bb>KK-1) j=KK;
      cc=arr1[zz]; if (cc>KK-1) j=KK;
      if (auxrule[aa][bb][cc][dd]==' ') j=KK;
      if (j==0) j=ascrule[aa][bb][cc][dd]-'0';
      arr1[yy+nc]=j;
      }
    break;
  case 'c':		/* test consistency everywhere */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx];
      bb=arr1[yy];
      cc=arr1[zz];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,c==ascrule[aa][bb][cc][dd]?7:4,1);
      }}
    m=1;
    break;
  case 'C':		/* test consistency for marked neighborhoods */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx];
      bb=arr1[yy];
      cc=arr1[zz];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,
        (c==ascrule[aa][bb][cc][dd] && auxrule[aa][bb][cc][dd]!=' ')?7:4,1);
      }}
    m=1;
    break;
  default: break;
      } /* end switch */
  if (m==0) for (k=0; k<nc; k++) {
    videocursor(0,ll+LROW,k);
    l=arr1[nc*ll+k]+'0';
    videoputc(l=='0'?'.':l,1);
    }
  m=0;
  }     /* end while */
}       /* end edline */

/* display a screen of evolution */
evolve() {int i, j;
  if (sc==7) {xevolve(); return;}
  videomode(COLGRAF);					/* erase the screen */
  videocursor(0,0,0);					/* top text line */
  scrstr(":");
  scrrul();
  asctobin();
  for (j=8; j<200; j++) videodot(j,AL-1,2);
  for (j=8; j<200; j++) {				/* evolve for 192 generations */
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    if (kbdst()) {kbdin(); break;}
    }
}

/* display a screen of evolution in Plantronics mode*/
xevolve() {int i, j, k, o, p; char l, m, n;
  videomode(COLGRAF);					/* erase the screen */
  outportb(0x3dd,0x20);
  videocursor(0,0,0);					/* top text line */
  scrstr(":");
  scrrul();
  asctobin();
  for (j=8; j<200; j++)					/* evolve for 192 generations */
    {
    for (i=0; i<AL/4; i++)
      {
      for (k=0, m=0, n=0; k<4; k++)
        {
	p=2*(4*i+k+AL);
        l=arr1[((p+j)/2)%AL];
        m=(m<<1)|(l&0x1); l>>=1; n=(n<<1)|(l&0x1);
        l=arr1[((p+1+j)/2)%AL];
        m=(m<<1)|(l&0x1); l>>=1; n=(n<<1)|(l&0x1);
        }
      o=80*(j/2)+i;
      if (j%2==0) pokeb(0xb800,o,m); else pokeb(0xba00,o,m);
      if (j%2==0) pokeb(0xbc00,o,n); else pokeb(0xbe00,o,n);
      }
    onegen(AL);
    if (kbdst()) {kbdin(); break;}
    }
}

/* copy ascrule over to binrul */
asctobin() {int i0, i1, i2, i3;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
      binrule[i0][i1][i2][i3]=ascrule[i0][i1][i2][i3]-'0';
}

/* evolution for one generation */
onegen(j) int j; {int i;
if (j>3) {
  arr2[0]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]];
  arr2[1]=binrule[arr1[j-1]][arr1[0]][arr1[1]][arr1[2]];
  for (i=2; i<j-1; i++) 
    arr2[i]=binrule[arr1[i-2]][arr1[i-1]][arr1[i]][arr1[i+1]];
  arr2[j-2]=binrule[arr1[j-4]][arr1[j-3]][arr1[j-2]][arr1[j-1]];
  arr2[j-1]=binrule[arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]];
} else if (j==3) {
  arr2[0]=binrule[arr1[1]][arr1[2]][arr1[0]][arr1[1]]; 
  arr2[1]=binrule[arr1[2]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[0]]; 
} else if (j==2) {
  arr2[0]=binrule[arr1[0]][arr1[1]][arr1[0]][arr1[1]]; 
  arr2[1]=binrule[arr1[1]][arr1[0]][arr1[1]][arr1[0]]; 
} else if (j==1) {
  arr2[0]=binrule[arr1[0]][arr1[0]][arr1[0]][arr1[0]]; 
}
for (i=0; i<j; i++) arr1[i]=arr2[i];
}

/* generate a random line of cells in arr1 */
ranlin() {int i, c;
for (i=0; i<AL; i++) {
  if (i%8 == 0) c=rand();
  arr1[i]=c%KK; c/=KK;}
}

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
  for (j=0; j*SL<AL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,LROW+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* tutorial and Help screen */
tuto() {
    videomode(T80X25);
    videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V. McIntosh, G. Cisneros S.>");
    videocursor(0,4,0);
    scrstr("           **** LIFE in One Dimension ****");
    videocursor(0,6,0);
    scrstr("Three States - Black(0), Cyan(1) Magenta(2).");
    videocursor(0,8,0);
    scrstr("3 half neighbors - two on each side, four altogether.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,14,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,15,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,17,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,21,0);
    scrstr("Now, ... press any key to continue.");
}

/* rule menu */
rmenu() {
    videocursor(0,CR-4,0); scrstr("      000000000111111111222222222");
    videocursor(0,CR-3,0); scrstr("      000111222000111222000111222");
    videocursor(0,CR-2,0); scrstr("      012012012012012012012012012");
    videocursor(0,CR,0);   scrstr("Rule: "); kscrrul();
    videocursor(0,CR,42);
    printf("      ");
    videocursor(0,CR,42);
    if (istot()==1) printf("(T-%2d)",totonu(0));
    videocursor(0,CM,0);
    scrstr("    r(rule), l(line), #(stored), g(graph), q(quit), t(prob),");
    videocursor(0,CM+1,0);
    scrstr("         x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,CM+2,0);
    scrstr("         @nn(tot/rule), $nn(12 tot/rules), T(ed tot/rule).");
    videocursor(0,CM,0);
    }

/* totalistic rule menu*/
tmenu() {
    videocursor(0,CR-3,50); scrstr("      0...1...2");
    videocursor(0,CR-2,50); scrstr("rule: "); tscrrul();
    videocursor(0,CR-2,56);
    }

/* line menu */
lmenu() {int i, j;
  for (j=0; 40*j<AL; j++) {
    videocursor(0,CL+j,0);
    for (i=0; i<40; i++) videoputc('0'+arr1[40*j+i],1);
    }
  videocursor(0,CM,0);
  }

/* display Wolfram rule number */
wmenu(n) int n;
{char xx[6]; int i, nn;  
    nn=sprintf(xx,"%5d",n);
    videocursor(0,2,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,2,40); }

/* display rule number */
xmenu(n) int n;
{char xx[4]; int i, nn;  
    nn=sprintf(xx,"%3d",n);
    videocursor(0,1,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,1,40); }

/* clear screen space for rule numbers */
xblnk() {
  videocursor(0,CR-4,40); scrstr("     ");
  videocursor(0,CR-3,40); scrstr("     ");
  videocursor(0,CR,0); }

/* clear auxrule to blanks */
auxblnk() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  auxrule[i0][i1][i2][i3]=' ';
}

/* clear ascrule to zeroes */
asczero() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]='0';
}

/* copy saved rule #n into active rule */
xtoasc(n) int n; { int i;
xmenu(n+1);
for (i=0; i<TS; i++) trule[i]=ixrule[n][i];
totoasc();
}

/* change totalistic rule to general rule */
totoasc() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=trule[i0+i1+i2+i3];
}

/* change decimal totalistic rule to sum values */
nutoto(x) int x; {int i; for (i=0; i<TS; i++) {trule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
 r=0; if (i<TS) r=(trule[i]-'0')+KK*totonu(i+1);
  return r; }

/* test whether a rule is totalistic */
int istot() {int i0, i1, i2, i3, l;
    l=1;
    trule[0]=ascrule[0][0][0][0];
    trule[1]=ascrule[1][0][0][0];
    trule[2]=ascrule[2][0][0][0];
    trule[3]=ascrule[2][1][0][0];
    trule[4]=ascrule[2][2][0][0];
    trule[5]=ascrule[2][2][1][0];
    trule[6]=ascrule[2][2][2][0];
    trule[7]=ascrule[2][2][2][1];
    trule[8]=ascrule[2][2][2][2];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    if (ascrule[i0][i1][i2][i3]!=trule[i0+i1+i2+i3]) l=0;      
    return l; }

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* display the rule number on the screen */
scrrul() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) 
    videoputc(ascrule[i0][i1][i2][i3],1);      
  }

/* display the rule number (base 27) on the screen */
kscrrul() {int i, i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
    {
    i=KK*(KK*(ascrule[i0][i1][i2][0]-'0')+
      ascrule[i0][i1][i2][1]-'0')+ascrule[i0][i1][i2][2];
    if (i>'9') i+='A'-'9';
    videoputc(i,1);      
    }
  }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1, i2, i3;
  i=6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    {
    videocursor(0,5,i++);
    videocattr(0,ascrule[i0][i1][i2][i3],auxrule[i0][i1][i2][i3]==' '?3:6,1);
    }
  }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
woruno(i,j) int i, j; {
char xx[12];
int  k, l, i0, i1, i2, i3;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
  for (i2=KK-1; i2>=0; i2--)
  for (i3=KK-1; i3>=0; i3--)
    k=KK*k+(ascrule[i0][i1][i2][i3]-'0');
  k=sprintf(xx,"%10u",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

/* display totalistic rule number by sum */
tscrrul() {int i; for (i=0; i<TS; i++) {videoputc(trule[i]); } }

/* end */
