
/* nxlcau32.c				*/
/* (3,2) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1990		*/
/*    Copyright (C) 1994		*/
/*    Copyright (C) 1995		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 14 June 1987 - individual cycles of evolution [HVM]		*/
/* 17 June 1987 - p-adic representation in plane [HVM]		*/
/* 22 June 1987 - 2 speed gliders via 16 transitions [HVM]	*/
/* 26 June 1987 - push, pop the rule [HVM]			*/
/* 26 June 1987 - conserve position of rule cursor [HVM]	*/
/* 27 June 1987 - incremental rule construction [HVM]		*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB41.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 24 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN31.C for option 'd' [HVM]		*/
/* 10 April 1988 - include rule engineering from LCA41 [HVM]	*/
/* 11 March 1989 - option 'x' in trule [HVM]			*/
/* 17 September 1989 - directory for demonstration rules [HVM]	*/
/* 5 October 1989 - modified for radius 1/2 [HVM]		*/
/* 10 February 1990 - Garden of Eden option [HVM]		*/

// # include <bdos.h>
# include "nxlcau32.h"
# include "rec.h"

# define LROW		 9	/* top row of line panel  */
# define DROWS		20	/* rows in demo menu	  */
# define DCOLS		32	/* second col in demo m.  */

# define NX          	14	/* number of sample rules */

char ixrule[NX][TS]= {

	"000101011",	/*  1 */
	"000101101",	/*  2 */
	"000101101",	/*  3 */
	"000101101",	/*  4 */
	"000101101",	/*  5 */
	"000101111",	/*  6 */
	"002011220",	/*  7 gridwork */
	"000110111",	/*  8 */
	"001011111",	/*  9 */
	"122201202",	/* 10 class iv */
	"010110101",	/* 11 */
	"011010011",	/* 12 */
	"200011202",	/* 13 colored snow */
	"110100101"	/* 14 */

	};

parneih    amat;
parneih    bmat;
parneih    cmat;
parneih    fmat;
parneih    gmat;
parneih    hmat;
parneih    xmat;

// twoparn    smat;
// twoparn    tmat;
// twoparn    umat;
// twoparn    vmat;

double stt, sta[5][K3+1];
double blpr[KK];			/* probability from blocks	*/

Inst 	xprg[10][XLEN];			
Inst   	rprg[16384];			/* rec program array */
char   	cstr[CLEN];			/* console rec program */
char   	cprg[CLEN];			/* edit stored program */
char	idef [1][30] = {"(z)!100!0r:;)1r2r(0r:;);)a"};

extern int rec_c(char *z, Inst rprg, int l, struct fptbl dtbl[]);

char   tabl[20][SL];			/* workspace to construct table */
char   ascrule[KK][KK][KK][KK][KK];	/* ASCII transition values */
char   auxrule[KK][KK][KK][KK][KK];	/* auxiliary transition table */
char   prule1[8], prule2[8];		/* product rule specification */
char   tarule[TS];
char   ttrule[TS];
char   strule[KK][SS];
char   sarule[KK][SS];
int    binrule[KK][KK][KK][KK][KK];
char   arr1[AL], arr2[AL], arr3[AL];
int    garry[AL];
int    ru, ru0, ru1, ru2, ru3;		/* rule cursor */
int    co=1, dn=1;			/* connectivity, dimension */
int    li, lj;				/* cell pointer */
int    di, dj;				/* demo pointer */
int    sc;				/* screen colors */
int    rulptr;				/* rule pd ptr */
int    ix0, iy0;			/* origin for pen moves */
double wmul[KK], wmvl[KK];		/* left mass point */
double wmur[KK], wmvr[KK];		/* right mass point */

int    cc, dd;			/* radius of de Bruijn ring */

# include "nxlcaukr.c"


lcau32main() {
int  i, j, i0, i1, i2, i3, i4;
int  more='r';
char a, b, c;

for (i=0; i<TS; i++) ttrule[i]='0';

ru=6; ru0=0; ru1=0; ru2=0; ru3=0;
li=SL/2; lj=0;					/* line editor cursor */
sc=0;

    videopalette(WHCYMAG);			/* white/cyan/magenta */

    tuto();
    while (!kbdst()) rand();			/* wait for keypress */
    kbdin();					/* ignore it */
    videomode(T80X25);
    videoscroll(0,0,CR,71,0,3);			/* menu on blue background */
    videoscroll(CL+8,0,24,71,0,3);
    i=(rand()%NX);
    di=i%DROWS;
    dj=(i/DROWS==1?DCOLS:1);
    xtoasc(i);
    auxblnk();
    ranlin();					/* random initial array */

    while (more!='n') {				/* execute multiple runs */
    rmenu();
    lmenu();
    while (0<1) {				/* set up one run */
    c=kbdin();
    if (c=='g') break;					/* go draw graph */
    if (c=='q') more='n';				/* quit for good */
    if (more=='n') break;
    switch (c) {
        case '!': sc=7; break;
	case '*': sc=2; break;				/* white on red */
	case '+': sc=1; break;				/* white screen */
	case '-': sc=0; break;				/* blue screen */
	case '@':					/* numbered tot rule */
	    nutoto(numin(0));
	    totoasc();
	    rmenu();
	    videocursor(0,CM,0);
	    break;
	case '$':					/* dozen totalistics */
	    j=numin(0);
	    for (i=0; i<12; i++) {
	      nutoto(j+i);
	      totoasc();
	      ranlin();
	      evolve();
	      }
            videomode(T80X25);
	    rmenu();
	    lmenu();
	    break;
	case 'T':					/* totalistic rule */
	    xblnk();
	    tmenu();
	    edtrule();
	    totoasc();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++)
	      ascrule[i0][i1][i2][i3][i4]=ttrule[i0+i1+i2+i3+i4];
	    videocursor(0,CM,0);
	    rmenu();
	    break;
	case 't':					/* probability calculation */
	    edtri();
	    rmenu();
	    lmenu();
	    break;
        case 'r':					/* edit rule */	
	    xblnk();
	    edrule();
	    videocursor(0,CM,0);
	    rmenu();
	    break;
        case 'l':					/* edit cell string */
	    xblnk();
	    edline(8,40);
	    videocursor(0,CM,0);
	    lmenu();
	    break;
        case '#':					/* read stored rule */
	    xmenu(NX);
	    xtoasc(lim(1,dmenu(),NX)-1);
	    rmenu();
            break;
	case 'D':					/* run through samples */
	    for (i=0; i<NX; i++) {
	      xmenu(i);
	      xtoasc(i);
	      ranlin();
	      evolve();
	      }
	    videomode(T80X25);
	    rmenu();
	    break;
	case 'd':					/* de Bruijn diagram */
	    edijn();
	    rmenu();
	    lmenu();
	    break;
	case 'e':					/* Garden of Eden */
//	    edard();
	    rmenu();
	    lmenu();
	    break;
        case 'u':					/* sparse init arry */
	    xblnk();
	    for (i=0; i<AL; i++) arr1[i]=0;
	    arr1[AL/4]=1;
            arr1[AL/2]=1;
            arr1[(3*AL)/4]=1;
            arr1[(3*AL)/4+2]=1;
	    lmenu();
            break;
	case 'w':					/* Wolfram rule # */
	    i=numin(0);
	    wmenu(i);
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++)
	      {ascrule[i0][i1][i2][i3][i4]='0'+i%KK; i/=KK;}
	    rmenu();
	    break;
	case 'X':					/* random rule */
	    xblnk();
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++) {
	      if ((KK*(KK*(KK*i0+i1)+i2)+i3)%4 == 0) i=rand();
	      ascrule[i0][i1][i2][i3][i4]='0'+i%KK;
	      i/=KK;
	    }
	    rmenu();
	    break;
	case 'x':					/* random rule */
	    xblnk();
	    i=rand();
	    for (i0=0; i0<KK; i0++) {
	    for (i1=0; i1<KK; i1++) {
	    for (i2=0; i2<KK; i2++) {
	    for (i3=0; i3<KK; i3++) {
	    for (i4=0; i4<KK; i4++) {
	      if (i == 0) i=rand();
	      if (auxrule[i0][i1][i2][i3][i4]==' ') {
		ascrule[i0][i1][i2][i3][i4]='0'+i%KK;
		i/=KK;
		}
	      }}}}
	    rmenu();
	    break;
	case 'y':					/* random line */
	    xblnk();
	    ranlin();
            lmenu();
	    break;
	case 'Y':					/* symmetrize rule */
	    for (i0=0; i0<KK; i0++)
	    for (i1=0; i1<KK; i1++)
	    for (i2=0; i2<KK; i2++)
	    for (i3=0; i3<KK; i3++)
	    for (i4=0; i4<KK; i4++) {
	    ascrule[i4][i3][i2][i1][i0]=ascrule[i0][i1][i2][i3][i4];      
	    }
	    break;
	case 'B':					/* begin barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[0][a-'0'][b-'0'][a-'0'][a-'0']=a;
	    ascrule[1][a-'0'][b-'0'][a-'0'][a-'0']=a;
	    ascrule[2][a-'0'][b-'0'][a-'0'][a-'0']=a;
	    ascrule[3][a-'0'][b-'0'][a-'0'][a-'0']=a;
	    rmenu();
	    break;
	case 'E':					/* end barrier */
	    a=kbdin();
	    b=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][0][a-'0']=b;
	    ascrule[a-'0'][a-'0'][a-'0'][1][a-'0']=b;
	    ascrule[a-'0'][a-'0'][a-'0'][2][a-'0']=b;
	    rmenu();
	    break;
	case 'L':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0'][a-'0']=c;
	    rmenu();
	    break;
	case 'R':					/* left glider link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0'][a-'0']=a;
	    rmenu();
	    break;
	case 'S':					/* still life link */
	    a=kbdin();
	    b=kbdin();
	    c=kbdin();
	    ascrule[a-'0'][a-'0'][a-'0'][c-'0'][a-'0']=b;
	    rmenu();
	    break;
	case '.':					/* one cycle of evolution */
	    asctobin();
	    onegen(AL);
	    lmenu();
	    break;
	case '=':
	    for (i=1; i<8;  i++) {
	    for (j=0; j<40; j++) arr1[40*i+j]=arr1[j];}
	    lmenu();
	    break;
	case '~':
	    for (i=1; i<16;  i++) {
	    for (j=0; j<20; j++) arr1[20*i+j]=arr1[j];}
	    lmenu();
	    break;
        default: break;
        }
    }
    if (more=='n') break;
    do {
    evolve();
    videocursor(0,0,0);
    scrstr("?");
    videocursor(0,0,34);
    scrstr("y/n/cr");
    more=kbdin();
    } while (more=='\015');
    videomode(T80X25);					/* reset the screen */
    if (more=='n') break;
    }
  videomode(T80X25);}   
} 

/* list of demonstration rules */
int dmenu() {int ii, jj; char c;
  videomode(T80X25);
videocursor(0,0,0); printf("Demonstration rules for LCAU3H");
videocursor(0,1,0); printf(" 1 - totalistic class ii");
videocursor(0,2,0); printf(" 2 - strips");
videocursor(0,3,0); printf(" 3 - slow growth");
videocursor(0,4,0); printf(" 4 -");
videocursor(0,5,0); printf(" 5 - Fisch's cyclic eater");
videocursor(0,6,0); printf(" 6 - trinity = sum mod 3");
videocursor(0,7,0); printf(" 7 - red triangles on blue");
videocursor(0,8,0); printf(" 8 - macrocells");
videocursor(0,9,0); printf(" 9 - drizzle");
videocursor(0,10,0); printf("10 - zigzags");
videocursor(0,11,0); printf("11 - more triangles");
videocursor(0,12,0); printf("12 - triangles");
videocursor(0,13,0); printf("13 - fine grained triangles");
videocursor(0,14,0); printf("14 - dotted triangles");
videocursor(0,15,0); printf("15 - more macrocells");
videocursor(0,16,0); printf("16 - class ii on red bgrnd");
videocursor(0,17,0); printf("17 - stochastic not doubly");
videocursor(0,18,0); printf("18 - <small> variance");
videocursor(0,19,0); printf("19 - small variance #2");
videocursor(0,20,0); printf("20 - red and black triangles");
videocursor(0,1,DCOLS); printf("21 - reversible = 201201021 ");
videocursor(0,2,DCOLS); printf("22 - perverse rule");
videocursor(0,3,DCOLS); printf("23 - mostly blue triangles");
videocursor(0,4,DCOLS); printf("24 - good mixture of cells ");
videocursor(0,5,DCOLS); printf("25 - reversible = 212000121");
videocursor(0,6,DCOLS); printf("26");
videocursor(0,7,DCOLS); printf("27");
videocursor(0,8,DCOLS); printf("28");
videocursor(0,9,DCOLS); printf("29");
videocursor(0,10,DCOLS); printf("30");
videocursor(0,11,DCOLS); printf("31");
videocursor(0,12,DCOLS); printf("32");
videocursor(0,13,DCOLS); printf("33");
videocursor(0,14,DCOLS); printf("34");
videocursor(0,15,DCOLS); printf("35");
videocursor(0,16,DCOLS); printf("36");
videocursor(0,17,DCOLS); printf("37");
videocursor(0,18,DCOLS); printf("38");
videocursor(0,19,DCOLS); printf("39");
videocursor(0,20,DCOLS); printf("40");
videocursor(0,22,0); printf("move cursor with arrows, select with <cr>");
videocursor(0,23,0); printf("position at top with <, bottom with >.");
  ii=DROWS; jj=DCOLS; c='!'; /* ii = max # rows, jj = second column */
  do {
    videocursor(0,di+1,dj);
    while (!kbdst()) videocursor(0,di+1,dj);
    c=kbdin();
      switch (c) {
      case '\013': case '\310': if (di==0) di=ii-1; else di--; break; /* up arrow */
      case '\012': case '\320': if (di==ii-1) di=0; else di++; break; /* down arrow */
      case '\014': case '\315': case ' ': dj=(dj==1?jj:1); break;     /* forward */
      case '\010': case '\313': dj=(dj==1?jj:1); break;		      /* backward */
      case '<': di=0; break;					      /* home */
      case '>': di=ii-1; break;					      /* bottom */
      default: break;
      }
    } while (c!='\015');
  videomode(T80X25);
  return(di+(dj==1?0:ii)+1);
}

/* edit the rule */
edrule() {char c;
  while (0<1) {
    cscrrul();
    videocursor(0,5,ru);
    c=kbdin();
    if (c=='\015') break;				/* carriage return exits */
    switch (c) {
    case '0': case '1':	case '2':			/* state */
      ascrule[ru0][ru1][ru2][ru3][ru3]=c;
      videocattr(0,c,3,1);
      rupl();
      break;
    case '\011':					/* tab = next quad */
      ru3=0; ru2=0; ru1=0; ru0++;
      if (ru0==KK) ru0=0;
      ru=KK*KK*KK*ru0+6;
      break;
    case ' ': case '\315': rupl(); break;				 /* advance */
    case '\010': case '\313': rumi(); break;				 /* backspace */
    case '\013': case '\310': auxrule[ru0][ru1][ru2][ru3][ru3]='+'; rupl(); break; /* up arrow */
    case '\012': case '\320': auxrule[ru0][ru1][ru2][ru3][ru3]=' '; rupl(); break; /* down arrow */
    case 'z': asczero();
    default: break;
    }
  }
}

/* advance the rule cursor */
rupl() {
  ru3++;
  if (ru3==KK) {ru3=0; ru2++;}
  if (ru2==KK) {ru2=0; ru1++;}
  if (ru1==KK) {ru1=0; ru0++;}
  if (ru0==KK) {ru0=0; }
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* retract the rule cursor */
rumi() {
  if (ru3!=0) ru3--; else {ru3=KK-1;
  if (ru2!=0) ru2--; else {ru2=KK-1;
  if (ru1!=0) ru1--; else {ru1=KK-1;
  if (ru0!=0) ru0--; else {ru0=KK-1;
     }}}}
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* edit totalistic rule */
edtrule() {char c; int  i, k;
  i=0;
  while (i<TS) {
    c=ttrule[i];
    videocursor(0,3,56+i);
    videocattr(0,c,3,1);
    c=kbdin();
    if (c=='\015') break;
    switch (c) {
      case '0': case '1': case '2':		/* state */
        ttrule[i]=c;
        videocattr(0,c,CR,1);
        i++;
        break;
      case ' ': case '\315':			/* space = advance */
        i++;
        break;
      case '\010': case '\313':			/* backspace */
        if (i!=0) i--;
	break;
      case 'x':					/* random selection */
	for (i=0; i<TS; i++) {
	  if (i%8==0) k=rand();
	  ttrule[i]='0'+k%KK; k/=KK;}
	i=0;
	tscrrul();
	break;
      default: break;
      }
  }
}

/* edit the line of cells */
edline(nr,nc) int nr, nc; {
char c;
int  i, j, k, l, m;
int  aa, bb, cc, dd;
int  ww, xx, yy, zz;
int  ll;

  m=0; /* refresh trigger */
  cscrrul();
  videocursor(0,19,0);
  scrstr("insert states with 0, 1; move cursor");
  videocursor(0,20,0);
  scrstr("with keyboard arrows, space or backspace.");
  videocursor(0,21,0);
  scrstr("carriage return exits");
  while (0<1) {
  ll=lj;
  videocursor(0,lj+LROW,li);
  c=kbdin();
  if (c == '\015') {videoscroll(19,0,24,70,0,3); break;}
  switch (c) {
  case '0': case '1': case '2':				/* enter state */
    arr1[nc*lj+li]=c-'0';
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\012': case '\320':				/* downwards */
    lj++; if (lj>=nr) lj=0;
    break;
  case '\013': case '\310':				/* upwards */
    if (lj>0) lj--; else lj=nr-1;
    break;
  case '\014': case '\315': case ' ':			/* forward */
    li++; if (li>=nc) {li=0; lj++;} if (lj>=nr) lj=0;
    break;
  case '\010': case '\313':				/* backward */
    if (li>0) li--;
      else {li=nc-1; if (lj>0) lj--; else lj=nr-1;}
    break;
  case '<':						/* left margin */
    li=0;
    break;
  case '>':						/* right margin */
    li=nc-1;
    break;
  case 'z':						/* clear single row */
    for (k=0; k<nc; k++) arr1[nc*lj+k]=0;
    break;
  case 'Z':						/* clear full array */
    for (k=0; k<AL; k++) arr1[k]=0;
    lmenu();
    break;
  case '^':						/* copy line above */
    for (k=0; k<nc; k++)
      arr1[nc*lj+k]=arr1[nc*(lj-1)+k];
    break;
  case 'x':						/* unmark transitions */
    auxblnk();
    cscrrul();
    break;				
  case 'q':						/* clean up the area */
    ulmenu(3);
    break;
  case '=':		/* insert point transition in ascrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    c=arr1[yy+nc]+'0';
    ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]=c;
    rmenu(); 
    cscrrul();
    break;
  case '+':		/* set marker for transition in auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]='+';
    rmenu(); 
    cscrrul();
    break;
  case '-':		/* remove marker for transition from auxrule */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]=' ';
    rmenu(); 
    cscrrul();
    break;
  case '*':		/* insert transitions for whole line */
    if (lj==0) break;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      c=arr1[yy+nc]+'0';
      ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]=c;
      auxrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]=c;
      }
    rmenu(); 
    cscrrul();
    break;
  case '.':		/* point evolution */
    if (lj==0) break;
    yy=li+nc*(lj-1);
    xx=li==0?yy+nc-1:yy-1;
    zz=li==nc-1?yy-nc+1:yy+1;
    arr1[yy+nc]=ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]-'0';
    break;
  case '?':		/* evolution of whole line */
    if (lj==nr-1) {for (i=0; i<AL-nc; i++) arr1[i]=arr1[i+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      arr1[yy+nc]=ascrule[arr1[ww]][arr1[xx]][arr1[yy]][arr1[zz]][arr1[zz]]-'0';
      }
    break;
  case '/':		/* conditional evolution */
    if (lj==nr-1) {for (k=0; k<AL-nc; k++) arr1[k]=arr1[k+nc]; lj--; lmenu(); }
    lj++; ll=lj;
    for (i=0; i<nc; i++) {
      j=0;
      yy=i+nc*(lj-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx]; if (aa>KK-1) j=KK;
      bb=arr1[yy]; if (bb>KK-1) j=KK;
      cc=arr1[zz]; if (cc>KK-1) j=KK;
      if (auxrule[aa][bb][cc][dd]==' ') j=KK;
      if (j==0) j=ascrule[aa][bb][cc][dd][dd]-'0';
      arr1[yy+nc]=j;
      }
    break;
  case 'c':		/* test consistency everywhere */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx];
      bb=arr1[yy];
      cc=arr1[zz];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,c==ascrule[aa][bb][cc][dd][dd]?7:4,1);
      }}
    m=1;
    break;
  case 'C':		/* test consistency for marked neighborhoods */
    for (i=0; i<nc; i++) {
      videocursor(0,LROW,i);
      k=arr1[i];
      videocattr(0,k==0?'.':k+'0',7,1);
      }
    for (j=1; j<nr;  j++) {
    for (i=0; i<nc; i++) {
      yy=i+nc*(j-1);
      xx=i==0?yy+nc-1:yy-1;
      zz=i==nc-1?yy-nc+1:yy+1;
      aa=arr1[xx];
      bb=arr1[yy];
      cc=arr1[zz];
      c=arr1[yy+nc]+'0';
      videocursor(0,j+LROW,i);
      videocattr(0,c=='0'?'.':c,
        (c==ascrule[aa][bb][cc][dd][dd] && auxrule[aa][bb][cc][dd][dd]!=' ')?7:4,1);
      }}
    m=1;
    break;
  default: break;
      } /* end switch */
  if (m==0) for (k=0; k<nc; k++) {
    videocursor(0,ll+LROW,k);
    l=arr1[nc*ll+k]+'0';
    videoputc(l=='0'?'.':l,1);
    }
  m=0;
  }     /* end while */
}       /* end edline */

/* calculate index, or rather the neighbors, in edline. */
linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
  if (i==0) xx[0]=x+nc-1; else xx[0]=x-1;
  if (i==nc-1) xx[2]=x-nc+1; else xx[2]=x+1;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}

/* display a screen of evolution */
evolve() {int i, j;
  if (sc==7) {xevolve(); return;}
//  videomode(COLGRAF);					/* erase the screen */
//  videocursor(0,0,0);					/* top text line */
//  scrstr(":");
//  scrrul();
  asctobin();
//  for (j=8; j<200; j++) videodot(j,AL-1,2);
  for (j=0; j<200; j++) {				/* evolve for 192 generations */
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
//    if (kbdst()) {kbdin(); break;}
    }
}

/* display a screen of evolution in Plantronics mode*/
xevolve() {int i, j, k, o, p; char l, m, n;
//  videomode(COLGRAF);					/* erase the screen */
//  outportb(0x3dd,0x20);
//  videocursor(0,0,0);					/* top text line */
//  scrstr(":");
//  scrrul();
  asctobin();
//  for (j=8; j<200; j++)				/* evolve for 192 generations */
    {
    for (i=0; i<AL/4; i++)
      {
      for (k=0, m=0, n=0; k<4; k++)
        {
	p=2*(4*i+k+AL);
        l=arr1[((p-j)/2)%AL];
        m=(m<<1)|(l&0x1); l>>=1; n=(n<<1)|(l&0x1);
        l=arr1[((p+1-j)/2)%AL];
        m=(m<<1)|(l&0x1); l>>=1; n=(n<<1)|(l&0x1);
        }
      o=80*(j/2)+i;
//      if (j%2==0) pokeb(0xb800,o,m); else pokeb(0xba00,o,m);
//      if (j%2==0) pokeb(0xbc00,o,n); else pokeb(0xbe00,o,n);
      }
    onegen(AL);
//    if (kbdst()) {kbdin(); break;}
    }
}

/* copy ascrule over to binrul */
asctobin() {int i0, i1, i2, i3, i4;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    for (i4=0; i4<KK; i4++)
      binrule[i0][i1][i2][i3][i4]=ascrule[i0][i1][i2][i3][i4]-'0';
}

/* evolution for one generation */
xonegen(j) int j; {int i;
if (j>5) {
  arr2[0]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]][arr1[2]];
  arr2[1]=binrule[arr1[j-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]];
  for (i=2; i<j-1; i++) 
    arr2[i]=binrule[arr1[i-2]][arr1[i-1]][arr1[i]][arr1[i+1]][arr1[i+2]];
  arr2[j-2]=binrule[arr1[j-4]][arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]];
  arr2[j-1]=binrule[arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]];
} else if (j==5) {
  arr2[0]=binrule[arr1[3]][arr1[4]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[1]=binrule[arr1[4]][arr1[0]][arr1[1]][arr1[2]][arr1[3]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[4]]; 
  arr2[3]=binrule[arr1[1]][arr1[2]][arr1[3]][arr1[4]][arr1[0]]; 
  arr2[4]=binrule[arr1[2]][arr1[3]][arr1[4]][arr1[0]][arr1[1]]; 
} else if (j==4) {
  arr2[0]=binrule[arr1[2]][arr1[3]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[1]=binrule[arr1[3]][arr1[0]][arr1[1]][arr1[2]][arr1[3]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[0]]; 
  arr2[3]=binrule[arr1[1]][arr1[2]][arr1[3]][arr1[0]][arr1[1]]; 
} else if (j==3) {
  arr2[0]=binrule[arr1[1]][arr1[2]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[1]=binrule[arr1[2]][arr1[0]][arr1[1]][arr1[2]][arr1[0]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[0]][arr1[1]]; 
} else if (j==2) {
  arr2[0]=binrule[arr1[0]][arr1[1]][arr1[0]][arr1[1]][arr1[0]]; 
  arr2[1]=binrule[arr1[1]][arr1[0]][arr1[1]][arr1[0]][arr1[1]]; 
} else if (j==1) {
  arr2[0]=binrule[arr1[0]][arr1[0]][arr1[0]][arr1[0]][arr1[0]]; 
}
for (i=0; i<j; i++) arr1[i]=arr2[i];
}

/* evolution for one generation */

onegen(j) int j; {int i, k, i0, i1, i2, i3, i4; long int ii;
switch (dn) {

case 1:
if (j>4) {
  arr2[0]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]][arr1[2]];
  arr2[1]=binrule[arr1[j-1]][arr1[0]][arr1[1]][arr1[2]][arr1[3]];
  for (i=2; i<j-2; i++) 
    arr2[i]=binrule[arr1[i-2]][arr1[i-1]][arr1[i]][arr1[i+1]][arr1[i+2]];
  arr2[j-2]=binrule[arr1[j-4]][arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]];
  arr2[j-1]=binrule[arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]];
} else if (j==4) {
  arr2[0]=binrule[arr1[2]][arr1[3]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[1]=binrule[arr1[3]][arr1[0]][arr1[1]][arr1[2]][arr1[3]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[3]][arr1[0]]; 
  arr2[3]=binrule[arr1[1]][arr1[2]][arr1[3]][arr1[0]][arr1[1]]; 
} else if (j==3) {
  arr2[0]=binrule[arr1[1]][arr1[2]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[1]=binrule[arr1[2]][arr1[0]][arr1[1]][arr1[2]][arr1[0]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[0]][arr1[1]]; 
} else if (j==2) {
  arr2[0]=binrule[arr1[0]][arr1[1]][arr1[0]][arr1[1]][arr1[0]]; 
  arr2[1]=binrule[arr1[1]][arr1[0]][arr1[1]][arr1[0]][arr1[1]]; 
} else if (j==1) {
  arr2[0]=binrule[arr1[0]][arr1[0]][arr1[0]][arr1[0]][arr1[0]]; 
}
if (co==1) for (ii=0; ii<j; ii++) arr1[(int)ii]=arr2[(int)(ii)];
if (co==0) for (ii=0; ii<j; ii++) arr1[(int)ii]=arr2[(int)((7*ii)%j)];
if (co==2) for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];}
if (co==3) for (ii=0; ii<j; ii+=3) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];
				 arr1[(int)ii+2]=arr2[(int)((7*ii+2)%j)];}
if (co==4) for (ii=0; ii<j; ii+=4) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];
				 arr1[(int)ii+2]=arr2[(int)((7*ii+2)%j)];
				 arr1[(int)ii+3]=arr2[(int)((7*ii+3)%j)];}

if (co==5) {for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((11*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((11*ii+1)%j)];}
	    for (ii=0; ii<j; ii+=8) {
		k=arr1[(int)ii];
		arr1[(int)ii]=arr1[(int)((ii+2)%j)];
		arr1[(int)((ii+2)%j)]=k;
		k=arr1[(int)((ii+1)%j)];
		arr1[(int)((ii+1)%j)]=arr1[(int)((ii+4)%j)];
		arr1[(int)((ii+4)%j)]=k;
		k=arr1[(int)((ii+3)%j)];
		arr1[(int)((ii+3)%j)]=arr1[(int)((ii+5)%j)];
		arr1[(int)((ii+5)%j)]=k;
                }
	    }
if (co==6) {for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((11*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((11*ii+1)%j)];}
	    for (ii=0; ii<j; ii+=16) {
		k=arr1[(int)ii];
		arr1[(int)ii]=arr1[(int)((ii+2)%j)];
		arr1[(int)((ii+2)%j)]=k;
		k=arr1[(int)((ii+1)%j)];
		arr1[(int)((ii+1)%j)]=arr1[(int)((ii+4)%j)];
		arr1[(int)((ii+4)%j)]=k;
		k=arr1[(int)((ii+3)%j)];
		arr1[(int)((ii+3)%j)]=arr1[(int)((ii+5)%j)];
		arr1[(int)((ii+5)%j)]=k;

		k=arr1[(int)((ii+8)%j)];
		arr1[(int)((ii+8)%j)]=arr1[(int)((ii+10)%j)];
		arr1[(int)((ii+10)%j)]=k;
		k=arr1[(int)((ii+9)%j)];
		arr1[(int)((ii+9)%j)]=arr1[(int)((ii+15)%j)];
		arr1[(int)((ii+15)%j)]=k;
		k=arr1[(int)((ii+11)%j)];
		arr1[(int)((ii+11)%j)]=arr1[(int)((ii+13)%j)];
		arr1[(int)((ii+13)%j)]=k;
		k=arr1[(int)((ii+12)%j)];
		arr1[(int)((ii+12)%j)]=arr1[(int)((ii+14)%j)];
		arr1[(int)((ii+14)%j)]=k;
		}
	    } break;

/* a radius 1 von Neumann neighborhood */

case 2: if (j<16384) break;
	  for (i0=0; i0<128; i0++) for (i1=0; i1<128; i1++) 
	    arr2[ix2(i0,i1)]=binrule
	      [arr1[ix2(i0,i1+1)]]
	      [arr1[ix2(i0-1,i1)]]
	      [arr1[ix2(i0,i1)]]
	      [arr1[ix2(i0+1,i1)]]
	      [arr1[ix2(i0,i1-1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* a tetrahedron with a cell in the center */

case 3: if (j<16384) break;
	  for (i0=0; i0<32; i0++)
	  for (i1=0; i1<32; i1++)
	  for (i2=0; i2<16; i2++)
	    arr2[ix3(i0,i1,i2)]=binrule
	      [arr1[ix3(i0,i1+1,i2+1)]]
	      [arr1[ix3(i0,i1-1,i2+1)]]
	      [arr1[ix3(i0,i1,i2)]]
	      [arr1[ix3(i0+1,i1,i2-1)]]
	      [arr1[ix3(i0-1,i1,i2-1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* four one-sided neighbors and their corner cell */

case 4: if (j<16384) break;
	  for (i0=0; i0<16; i0++) for (i1=0; i1<8; i1++) 
	  for (i2=0; i2<16; i2++) for (i3=0; i3<8; i3++) 
	    arr2[ix4(i0,i1,i2,i3)]=binrule
	      [arr1[ix4(i0+1,i1,i2,i3)]]
	      [arr1[ix4(i0,i1+1,i2,i3)]]
	      [arr1[ix4(i0,i1,i2,i3)]]
	      [arr1[ix4(i0,i1,i2+1,i3)]]
	      [arr1[ix4(i0,i1,i2,i3+1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* five one-sided neighbors without the corner cell */

case 5: if (j<16384) break;
	  for (i0=0; i0<8; i0++) for (i1=0; i1<8; i1++) 
	  for (i2=0; i2<8; i2++) for (i3=0; i3<8; i3++) 
	  for (i4=0; i4<4; i4++)
	    arr2[ix5(i0,i1,i2,i3,i4)]=binrule
	      [arr1[ix5(i0+1,i1,i2,i3,i4)]]
	      [arr1[ix5(i0,i1+1,i2,i3,i4)]]
	      [arr1[ix5(i0,i1,i2+1,i3,i4)]]
	      [arr1[ix5(i0,i1,i2,i3+1,i4)]]
	      [arr1[ix5(i0,i1,i2,i3,i4+1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

default: break;}
}

/* index five dimensions */
int ix5(i,j,k,l,m) int i, j, k, l, m; {
return (8+i)%8+ 8*( (8+j)%8+ 8*( (8+k)%8+ 8*( (8+l)%8+ 8*( (4+m)%4 ))));}

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
  for (j=0; j*SL<AL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,LROW+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* map the automaton state into the unit square */
pprob() {int i, j; float x, y, z, k;
i=AL/2; k=(float)KK; z=(float)((197/(KK*KK))*KK*KK);
x=0.0; for (j=0; j<30; j++) x=(x+(float)arr1[i+30-j])/k;
y=0.0; for (j=0; j<30; j++) y=(y+(float)arr1[i-29+j])/k;
videodot((int)(z*(1.0-y)),(int)(z*x),1);
}

/* set up a grid for the unit square */
pgrid() {int i, j, k, l;
k=KK*KK+1;
l=195/(k-1);
for (i=0; i<k; i++) {
for (j=0; j<k; j++) {
videodot(l*i,l*j,2);
}}
}

/* tutorial and Help screen */
tuto() {
    videomode(T80X25);
    videocursor(0,2,0);
    scrstr("<Copyright (C) 1987, 1988, 1989 - H.V.McIntosh, G.Cisneros S.>");
    videocursor(0,4,0);
    scrstr("             ** LIFE in One Dimension **");
    videocursor(0,6,0);
    scrstr("Three States - Black(0), Cyan(1), Magenta(2).");
    videocursor(0,8,0);
    scrstr("Leftsided neighbors - two cells per neighborhood.");
    videocursor(0,10,0);
    scrstr("Complete transition rule - random, edited, or stored.");
    videocursor(0,12,0);
    scrstr("Totalistic transition rule - random, edited, or stored.");
    videocursor(0,14,0);
    scrstr("Initial Cellular Array - random, edited, or patterned.");
    videocursor(0,16,0);
    scrstr("Submenus in options t(probabilities) and d(de Bruijn)");
    videocursor(0,17,0);
    scrstr("will be displayed in response to typing ?");
    videocursor(0,19,0);
    scrstr("Use any key to terminate a display in progress.");
    videocursor(0,22,0);
    scrstr("now, ... press any key to continue.");
}

/* rule menu */
rmenu() {
    videocursor(0,1,0);
    scrstr("      000111222");
    videocursor(0,2,0);
    scrstr("      012012012");
    videocursor(0,3,0);
    scrstr("Rule: ");
    scrrul();
    if (istot()==1) {videocursor(0,TRR,TRC); xmenu(totonu(0));}
    videocursor(0,5,0);
    scrstr("    r(rule), l(line), #(stored), g(graph), q(quit), t(prob),");
    videocursor(0,6,0);
    scrstr("           x(rand rule), y(rand line), u(unit line), d(deBruijn)");
    videocursor(0,7,0);
    scrstr("           @nn(tot/rule), $nn(12 tot/rules), T(ed tot/rule).");
    videocursor(0,5,0);
    }

/* totalistic rule menu*/
tmenu() {
    videocursor(0,TRR-2,TRC-6);
    scrstr("      totalistic rule");
    videocursor(0,TRR-1,TRC-6);
    scrstr("      0.1.2");
    videocursor(0,TRR,TRC-6);
    scrstr("rule: ");
    tscrrul();
    videocursor(0,TRR,TRC);
    }

/* plain line menu */
lmenu() {int i, j; char c;
  for (j=0; SL*j<AL; j++) {videocursor(0,LROW+j,0);
  for (i=0; i<SL; i++) {c='0'+arr1[SL*j+i]; videoputc(c=='0'?'.':c,1); } }
  videocursor(0,5,0); }

/* color line menu */
clmenu() {int c, i, j;
  for (j=0; SL*j<AL; j++) {
    for (i=0; i<SL; i++) {
      videocursor(0,LROW+j,i);
      c=arr1[SL*j+i];
      videocattr(0,c==0?'.':'0'+c,color(c),1);
      }}
  videocursor(0,5,0); }

/* get compatible color */
int color(i) int i; {
  switch (i) {
    case 0:  return 2;
    case 1:  return 3;
    case 2:  return 5;
    case 3:  return 7;
    default: return 6;
}}

/* display Wolfram rule number */
wmenu(n) int n;
{char xx[6]; int i, nn;  
    nn=sprintf(xx,"%5d",n);
    videocursor(0,2,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,2,40); }

/* display rule number */
xmenu(n) int n; {
char xx[6]; int i, nn;  
  nn=sprintf(xx,"%5u",n);
  videocursor(0,XRR,XRC);
  for (i=0; i<nn; i++) videoputc(xx[i],1);
  videocursor(0,XRR,XRC); }

/* clear screen space for rule number */
xblnk() {videocursor(0,XRR,XRC); scrstr("     "); videocursor(0,5,0); }

/* copy saved rule #n into active rule */
xtoasc(n) int n; {int i0, i1, i2, i3, i4;
//  xmenu(n+1);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
    ascrule[i0][i1][i2][i3][i4]=ixrule[n][(((i0*KK+i1)*KK+i2)*KK+i3)*KK+i3];
  }

/* change totalistic rule to general rule */
void totoasc() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  ascrule[i0][i1][i2][i3][i4]=ttrule[i0+i1+i2+i3+i4];
}

/* change decimal totalistic rule to sum values */
void nutoto(x) int x; {int i;
for (i=0; i<TS; i++) {ttrule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
  if (i<TS) r=(ttrule[i]-'0')+KK*totonu(i+1); else r=0;
  return r; }

/* test whether a rule is totalistic */
int istot() {int i0, i1, i2, i3, i4, l, trule[TS];
    l=1;
    trule[0]= ascrule[0][0][0][0][0];
    trule[1]= ascrule[1][0][0][0][0];
    trule[2]= ascrule[2][0][0][0][0];
    trule[3]= ascrule[2][1][0][0][0];
    trule[4]= ascrule[2][2][0][0][0];
    trule[5]= ascrule[2][2][1][0][0];
    trule[6]= ascrule[2][2][2][0][0];
    trule[7]= ascrule[2][2][2][1][0];
    trule[8]= ascrule[2][2][2][2][0];
    trule[9]= ascrule[2][2][2][2][1];
    trule[10]=ascrule[2][2][2][2][2];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    for (i4=0; i4<KK; i4++) {
    if (ascrule[i0][i1][i2][i3][i4]!=trule[i0+i1+i2+i3+i4]) l=0;
    }
    return l; }

/* find the totalistic rule number of a known ascii totalistic rule */
void asctotot() {char trule[TS]; int i0, i1, i2, i3, i4;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
    for (i4=0; i4<KK; i4++)
      ttrule[i0+i1+i2+i3+i4] = ascrule[i0][i1][i2][i3][i4];
}

/* generate a random rule */
ranrul() {int i, i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  i=(random()&0xFF)%KK;
  ascrule[i0][i1][i2][i3][i4]=i+'0';
  }
}

/* generate a random protected rule */
ranprul() {int i0, i1, i2, i3, i4, i, j;
i=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  if (i%8 == 0) j=rand(); i++;
  if (auxrule[i0][i1][i2][i3][i4]==' ') ascrule[i0][i1][i2][i3][i4]='0'+j%KK;
  j/=KK;
  }
}

/* generate a random totalistic rule */
rantrul() {int i0, i, j;
i=0;
for (i0=0; i0<TS; i0++) {
  if (i%8 == 0) j=rand(); i++;
  ttrule[i0]=j%KK+'0'; j/=KK;
  }
}

/* generate a random line of cells in arr1 */
ranlin() {int i, c;
for (i=0; i<AL; i++) {
  if (i%8 == 0) c=rand();
  arr1[i]=c%KK; c/=KK;}
}

/* generate a random core for a planar array */
ranplacor() {int i, j;
  for (i=60; i<69; i++) for (j=60; j<69; j++)  arr1[ix2(i,j)] = random()%KK;}

/* generate a random core for a solid array */
ransolcor() {int i, j, k;
  for (i=14; i<19; i++) for (j=14; j<19; j++) for (k=7; k<10; k++) 
	 arr1[ix3(i,j,k)] = random()%KK;}

/* clear auxrule to blanks */
auxblnk() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  auxrule[i0][i1][i2][i3][i4]=' ';
}

/* clear ascrule to zeroes */
asczero() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  ascrule[i0][i1][i2][i3][i4]='0';
}

/* clear auxrule to zeroes */
auxzero() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  auxrule[i0][i1][i2][i3][i4]=0;
}

/* change semitotalistic rule to general rule */
sttoasc() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  ascrule[i0][i1][i2][i3][i4]=strule[i2][i0+i1+i3+i4];
}

/* clear ttrule to zeroes */
ttzero() {int i; for (i=0; i<TS; i++)  ttrule[i]='0';}

/* clear strule to zeroes */
stzero() {int i, j;
  for (i=0; i<KK+1; i++) for (j=0; j<SS-1; j++)  strule[i][j]='0';
}

/* clear gf3rule to zeroes */
gfzero() {}


/* generate a random line of cells in arr1 with assigned probabilities */
ranblin(p1,p2,p3) double p1, p2, p3; {
int    i, c, i1, i2;
double p;
  p=p1+p2+p3;
  i1=(int)((1000.0*p1)/p);
  i2=i1+(int)((1000.0*p2)/p);
  for (i=0; i<AL; i++) {
    c=random()%1000;
    if (c<i1) arr1[i]=0; else if (c<i2) arr1[i]=1; else arr1[i]=2;
    }
}

/* put a single i-cell on a line of j-cells */
void putionj(i,j) int i, j; 
{int k; for (k=0; k<AL; k++) arr1[k]=j; arr1[AL/2]=i;}

/* make all of arr1 equal to zero */
zerline() {int i; for (i=0; i<AL; i++) arr1[i]=0;}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* display the rule number on the screen */
scrrul() {int i0, i1, i2, i3, i4;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
    videoputc(ascrule[i0][i1][i2][i3][i4],1);
  }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1, i2, i3, i4;
  i=6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
  videocursor(0,3,i++);
  videocattr(0,ascrule[i0][i1][i2][i3][i4],auxrule[i0][i1][i2][i3][i4]==' '?3:6,1);
  } }

// /* display totalistic rule number by sum */
// tscrrul() {int i;
//   for (i=0; i<TS; i++) {videoputc(ttrule[i],1); } }

/* change decimal totalistic rule to sum values */
nutowo1(x) int x; {int i;
for (i=0; i<TS; i++) {prule1[i]=x%2+'0'; x/=2;}}

/* change Wolfram rule number to decimal rule number */
int wotonu1(i) int i; {int r;
  if (i<TS) r=(prule1[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* change decimal number to Wolfram rule number */
nutowo2(x) int x; {int i;
for (i=0; i<TS; i++) {prule2[i]=x%2+'0'; x/=2;}}

/* change sum values to decimal totalistic rule */
int wotonu2(i) int i; {int r;
  if (i<TS) r=(prule2[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
woruno(i,j) int i, j; {
char xx[10];
int  k, l, i0, i1;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
    k=3*k+(ascrule[i0][i1]-'0');
  k=sprintf(xx,"%3d",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

void rule32(){
    ranrul();
//    xtoasc(((int)random()%NX));
    ranlin();					/* random initial array */
}

void savearry()  {int i; for (i=0; i<AL; i++) arr3[i]=arr1[i];}
void renewarry() {int i; for (i=0; i<AL; i++) arr1[i]=arr3[i];}

int  bdos(){}

/* end nxlcau32.c */
