
/* nxrijn32.c						*/
/* program for lcau32 option 'd'			*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 3 March 1988 - adapted from lca21.c to lca22.c	*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl41.c incorporated into rijn41.c	*/
/* 20 November 1989 - eliminated loop in kwait		*/

/*  Copyright (C) 1987	*/
/*  Copyright (C) 1988	*/
/*  Copyright (C) 1989	*/
/*  Copyright (C) 1990	*/
/*  Copyright (C) 1994	*/
/*  Harold V. McIntosh	*/

#include <math.h>
#import  "nxlcau32.h" 
#import  "nxrijn32.h" 
#import  <dpsclient/psops.h>
#import  <appkit/graphics.h>

extern NXColor colst[KK];

# define EROW		23	/* row for sample evolution */

# define JX  0.50  /* x-center, de Bruijn diagram	*/
# define JY  0.50  /* y-center, de Bruijn diagram	*/
# define RI 0.400  /* inner radius, de Bruijn diagram	*/
# define RO 0.425  /* outer radius, de Bruijn diagram	*/
# define NW    24  /* pause after so many lines		*/

extern int  dd;		/* # points in de Bruijn ring	*/
int  mc, nc, nl;	/* column & line control	*/
extern char arule[DS], arr1[AL], arr2[AL], arr3[AL];
extern char ascrule[KK][KK][KK][KK][KK];
extern int  binrule[KK][KK][KK][KK][KK];
extern int  ix0, iy0;				/* origin for pen moves */
extern int  dbno;				/* node in de Bruijn dgm */

char         aarry[KK][KK][KK][KK][KK];
char barry[KK][KK][KK][KK][KK][KK][KK][KK][KK];

double dcntr[DDN][2];

/* edit the de Bruijn diagram		*/
/* g = number of generation (0==2nd)	*/
/* s = shift distance (g==0)		*/
void edijn(g,s,p,r) int g, s, p, r; {

   switch(g+'a') {
//    case 'A': dd=KK*KK*KK*KK; ijn(dd,KK,2); sijn(0,1); break;
//    case 'B': dd=KK*KK*KK*KK; ijn(dd,KK,2); sijn(1,1); break;
//    case 'C': dd=KK*KK*KK*KK; ijn(dd,KK,2); zijn(dd,0,1); break;
//    case 'D': dd=KK*KK*KK*KK; ijn(dd,KK,2); zijn(dd,1,1); break;
//    case 'E': dd=KK*KK*KK*KK; ijn(dd,KK,2); zijn(dd,2,1); break;
//    case 'F': dd=KK*KK*KK*KK; ijn(dd,KK,2); zijn(dd,3,1); break;
//    case 'G': dd=KK*KK*KK*KK; ijn(dd,KK,2); zijn(dd,4,1); break;
    case 'G': xpass1(s); break;
    case 'H': ypass1(s); break;

    case 'a': switch (s+'g') {
      case 'c': bpass1(9,p,r); break;
      case 'd': bpass1(10,p,r); break;
      case 'e': apass1(0,p,r); break;
      case 'f': apass1(1,p,r); break;
      case 'g': apass1(2,p,r); break;
      case 'h': apass1(3,p,r); break;
      case 'i': apass1(4,p,r); break;
      case 'j': bpass1(11,p,r); break;
      case 'k': bpass1(12,p,r); break;
    default: break;} break;

    case 'b': switch (s+'g') {
      case 'c': bpass1(0,p,r); break;
      case 'd': bpass1(1,p,r); break;
      case 'e': bpass1(2,p,r); break;
      case 'f': bpass1(3,p,r); break;
      case 'g': bpass1(4,p,r); break;
      case 'h': bpass1(5,p,r); break;
      case 'i': bpass1(6,p,r); break;
      case 'j': bpass1(7,p,r); break;
      case 'k': bpass1(8,p,r); break;
      case '0': bpass1(13,p,r); break;
      case '1': bpass1(14,p,r); break;
    default: break;} break;

    default: break;
    } /* end switch */
//  }   /* end while  */
//  videopalette(WHCYMAG);
//  videomode(T80X25);
}

/* clear edijn screen and set up menu */
void clijn() {
  videomode(COLGRAF);
  videopalette(YELREGR);
  videocursor(0,0,0);
  scrrul();
  videocursor(0,3,0);
  printf("de Bruijn ring\n\n");
  printf("ab   -  0*,1*\n");
  printf("AB   -  full n*\n");
  printf("cdefg - (1,x)\n");
  printf("CDEFG - (all)\n");
  printf("h.l.p - (2,x)\n");
  printf("y - cycles 1-7 \n");
  printf("12345 - diagram\n");
  printf("+-   - pallette\n");
  printf("?/   - clr scrn\n");
  printf("<cr> - exit\n\n");
}

/* edit the Pass 4 and Pass 5 shift displays */
void edshg(g,s) int g, s; {
printf("edshg(%d,%d)\n",g,s);
    switch (shflev(g,s)) {
    case 0: apass6(); break;
//    case 1: bpass6(); break;
//    case 2: cpass6(); break;
//    case 3: dpass6(); break;
//    case 4: epass6(); break;
//    case 5: fpass6(); break;
    default: break;
    } /* end switch */
}

/* edit the Pass 4 and Pass 5 shift displays */
void edshf(g,s,c) int g, s, c; {
printf("edshf(%d,%d,%d)\n",g,s,c);
    switch (shflev(g,s)) {
    case 0: apass5(c); break;
//    case 1: bpass5(c); break;
//    case 2: cpass5(c); break;
//    case 3: dpass5(c); break;
//    case 4: epass5(c); break;
//    case 5: fpass5(c); break;
    default: break;
    } /* end switch */
}

/* edit the Pass 5 (sample loops) constant displays */
void edshc(g,c) int g, c; {
  switch(g) {
    case 0: apass5(c); break;
    case 1: bpass5(c); break;
    default: break;}
}

/* edit the Pass 4 and Pass 5 constant displays */
void edcst(s) int s; {
    switch(s) {
    case 0: apass6(); break;
    case 1: bpass6(); break;
//    case 2: cpass6(); break;
//    case 3: dpass6(); break;
//    case 4: epass6(); break;
//    case 5: fpass6(); break;
    default: break;
    } /* end switch */
}

int permut(k,n) int k, n; {
  switch (n) {
    case 0: break;
    case 1: k=(k+1)%KK; break;
    default: break;}
  return k;
}

/* - - - - - - - - d e   B r u i j n - - - - - - - - - - - - - - - - - - */

/* generate a de Bruijn diagram with m nodes, n links per node, color l */
void ijn(m,n,l) int m, n, l; {
int    i, j;
double ii, jj, nn, x, y, t, h;
printf("hello from ijn ... %3d, %3d, %3d\n",m,n,l);
nn=(double)(n);
t=6.28318/((double)(m));
h=0.5*t;
for (i=0; i<m; i++) {
  ii=(double)(i);
  for (j=0; j<n; j++) {
    jj=(double)(j);
    x=JX-RO*sin(ii*t+h);
    y=JY-RO*cos(ii*t+h);
    videoline(x,y,0);
    x=JX-RO*sin((nn*ii+jj)*t+h);
    y=JY-RO*cos((nn*ii+jj)*t+h);
    videoline(x,y,l);
    }
  }
}

/* insert a link into a de Bruijn diagram */
/* u - initial vertex     */
/* v - terminal vertex    */
/* l - color of link	  */

void lijnk(u,v,l) int u, v, l; {
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, w, t, sx, sy;

	if (l<KK) NXSetColor(colst[l]); else PSsetgray(NX_BLACK);
	r=10.3;
	w=3.33;
	ex=dcntr[u][0];
	wy=dcntr[u][1];
	x=dcntr[v][0];
	y=dcntr[v][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	sx=-w*sin(t);
	sy= w*cos(t);
	eks=x-r*cos(t)+sx;
	wye=y-r*sin(t)+sy;
	PSmoveto(ex+sx,wy+sy);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);
	PSmoveto(x+10.3,y);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSstroke();

   PSsetgray(NX_BLACK); 

   ex=dcntr[u][0];
   wy=dcntr[u][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",u);
   PSshow(s);

   ex=dcntr[v][0];
   wy=dcntr[v][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",v);
   PSshow(s);
   PSstroke();

}

/* generate a de Bruijn diagram showing ancestors of state k in color l */
void sijn(k,l) int k, l; {
int    i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
  if (k==ascrule[i0][i1][i2][i3][i4]-'0') 
	lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,l);
}

/* generate a de Bruijn diagram showing the (1,k) links in color l */
void zijn(m,k,l) int m, k, l; {
int    i, i0, i1, i2, i3, i4;
double th, x, y, t, h;
double ssin(), ccos();
t=6.28318/((double)(m));
h=0.5*t;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  i=ascrule[i0][i1][i2][i3][i4]-'0';
  if ((k==0&&i==i0)||(k==1&&i==i1)||(k==2&&i==i2)||(k==3&&i==i3)||(k==4&&i==i4)) {
    th=((double)(((i0*KK+i1)*KK+i2)*KK+i3)*t+h);
    x=JX-RI*ssin(th);
    y=JY-RI*ccos(th);
    videolyne(x,y,0);
    th=((double)(((i1*KK+i2)*KK+i3)*KK+i4)*t+h);
    x=JX-RO*ssin(th);
    y=JY-RO*ccos(th);
    videolyne(x,y,l);
    }	     /* end if    */
  }}}}} /* end for's */
}	     /* end zijn  */

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videolyne(x,y,l) double x, y; int l; {
int k, ax, ay, dx, dy, di, dj, x1, y1;
/* if (x<0.0) return; if (x>1.0) return */
/* if (y<0.0) return; if (y>1.0) return */
x1=(int)(199.0*(1.0-y));y1=(int)(199.0*x);
dx=x1-ix0;
dy=y1-iy0;
ax=dx>=0?dx:-dx;
ay=dy>=0?dy:-dy;
di=dx>=0?1:-1; if (dx==0) di=0;
dj=dy>=0?1:-1; if (dy==0) dj=0;

if (l>0) {if (ax!=0 || ay!=0) {
  if (ax>=ay) {for (k=0; k<=ax; k++) videodot(ix0+di*k,60+iy0+(k*dy)/ax,l);}
	 else {for (k=0; k<=ay; k++) videodot(ix0+(k*dx)/ay,60+iy0+dj*k,l);}
  } }
ix0=x1;
iy0=y1;
}

/* approximation to sine */
double ssin(x) double x; {
if (x<0.0) return(-ssin(-x));
while (x>=6.28318) x-=6.28318;
if (x>=3.142) return(-ssin(x-3.14159));
if (x>=1.571) return(ssin(3.14159-x));
return(x*(1.0-0.166*(x*x-0.05*x*x)));
}

/* approximation to cosine */
double ccos(x) double x; {double ssin(); return(ssin(x+1.57079));}

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videoline(x,y,l) double x, y; int l; {
if (l==0) PSmoveto(400.0*x,400.0*y);
else PSlineto(400.0*x,400.0*y);
}

/* find the level of de Bruijn diagram appropriate to g, s  */
int shflev(g,s) int g, s; {int  i; i=s<0?-s:s; return (i-1)/2;}

/* increment ix */ 
void incix(ix,n) int *ix, n; {int i; 
  for (i=0; i<n; i++) {
    ix[i]++;
    if (ix[i]>=KK) ix[i]=0; else return;
    }
}

/* selected node in ix */ 
void nodix(ix,n) int *ix, n; {int i, j; 
  j=dbno;
  for (i=0; i<n; i++) {ix[n-i-1]=j%KK; j/=KK;}
}

/* random number in ix */ 
void ranix(ix,n) int *ix, n; {int i, j; 
  j=random();
  for (i=0; i<n; i++) {ix[i]=j%KK; j/=KK;}
}

/* clear ix */ 
void zerix(ix,n) int *ix, n; {int i; for (i=0; i<n; i++) ix[i]=0;}

/* find index of nearest center to x, y (de Bruijn) */
int nearest(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<dd; i++) {
  ex=x-dcntr[i][0];
  wy=y-dcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/* ONE32.C */

/* Pass 1a analyzes all the configurations which fulfil (-1+l,1) */
void apass1(l,p,r) int l, p, r; {
int  ix[RR], m;
asctobin();
printf("RIJN32 - apass1: %d, %d, %d\n",l,p,r);
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++) {
  switch (l) {
    case 0: m=ix[0]; break;
    case 1: m=ix[1]; break;
    case 2: m=ix[2]; break;
    case 3: m=ix[3]; break;
    case 4: m=ix[4]; break;
    default: break;}
  aarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]]=
    binrule[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]]==m?'Y':'N';
  }
if (r) {apass2i(); apass2o();}
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void xpass1(l) int l; {
int  ix[RR];
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++) {
  aarry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]]=
    binrule[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]]==l?'Y':'N';
  }
apass2i();
apass2o();
}

/* Pass 2i flags all links with an incoming arrow */
/* Pass 2o flags all links with an outgoing arrow */
/* Then pass 3 discards all unflagged links */
/* Passes 2 and 3 alternate until no change is observed */

void apass2i() {
int i0, i1, i2, i3, i4, m;
do {
// printf(" Pass2i\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
if ((aarry[i0][i1][i2][i3][i4]&0x5F)=='Y')
  {for (m=0; m<KK; m++) aarry[i1][i2][i3][i4][m]|=0x20;}
} } while (apass3()!=0); }

void apass2o() {
int i0, i1, i2, i3, i4, m;
do {
// printf(" Pass2o\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
if ((aarry[i0][i1][i2][i3][i4]&0x5F)=='Y')
  {for (m=0; m<KK; m++) aarry[m][i0][i1][i2][i3]|=0x20;}
} } while (apass3()!=0); }

/* Pass 3 - erase flags, mark survivors, detect changes */

int apass3() {
int i0, i1, i2, i3, i4, l;
l=0;
// printf(" Pass3 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  switch (aarry[i0][i1][i2][i3][i4]) {
    case 'y': aarry[i0][i1][i2][i3][i4]='Y'; break;
    case 'Y': aarry[i0][i1][i2][i3][i4]='N'; l=1; break;
    case 'n': case 'N': aarry[i0][i1][i2][i3][i4]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 - print loops which remain */
void apass4() {
int i0, i1, i2, i3, i4;
int j0, j1, j2, j3, j4;
int k, l, m;
// printf(" pass4 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4;
l=0; m=0;
do {
        if (aarry[0][j1][j2][j3][j4]=='Y')
    {aarry[0][j1][j2][j3][j4]='y';
    k=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (aarry[1][j1][j2][j3][j4]=='Y')
    {aarry[1][j1][j2][j3][j4]='y';
    k=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=k;} }}
  } while (l==0);
l=0; m=0;
do {
        if (aarry[j0][j1][j2][j3][0]=='y')
   {aprf(j0,j1,j2,j3,0);
   aarry[j0][j1][j2][j3][0]='N';
   j0=j1; j1=j2; j2=j3; j3=0; m=1;}
  else {if (aarry[j0][j1][j2][j3][1]=='y')
   {aprf(j0,j1,j2,j3,1);
   aarry[j0][j1][j2][j3][1]='N';
   j0=j1; j1=j2; j2=j3; j3=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (aarry[j0][j1][j2][j3][0]=='Y')
   {aprf(j0,j1,j2,j3,0);
   aarry[j0][j1][j2][j3][0]='N';
   j0=j1; j1=j2; j2=j3; j3=0; m=1;}
  else {if (aarry[j0][j1][j2][j3][1]=='Y')
   {aprf(j0,j1,j2,j3,1);
   aarry[j0][j1][j2][j3][1]='N';
   j0=j1; j1=j2; j2=j3; j3=1; m=1;}
  else {l=1; if (m==1) kwait(0);}}
  } while (l==0);
}
}

/* print one link of the de Bruijn diagram */
aprf(i0,i1,i2,i3,i4) int i0, i1, i2, i3, i4; {
kwait(1);
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("%1d",i3);
printf("-");
printf("%1d",i4);
printf("  ");
lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,3);
}

/* Pass 5 - show evolution of sample loops */
void apass5(c) char c; {
int i0, i1, i2, i3, i4;
int j1, j2, j3, j4;
int hl;
int b[KK], irry[RR], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR); break;
    case 'r': ranix(irry,RR); break;
    case 'n': nodix(irry,RR-1);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];
	for (i=j=0; j<KK; j++)
		if (aarry[i0][i1][i2][i3][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[4]=b[l];
	break;
    case 'i': incix(irry,RR); break;
    default:  zerix(irry,RR); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];  i4=irry[4];
for (k=0; k<81; k++) {
  if (aarry[i0][i1][i2][i3][i4]=='Y') break;
  incix(irry,RR);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; i4=irry[4];
  }
hl=AL/2;
arr1[hl-2]=i0;
arr1[hl-1]=i1;
arr1[hl]=i2;
arr1[hl+1]=i3;
arr1[hl+2]=i4;
j1=i1; j2=i2; j3=i3; j4=i4;
for (k=hl-3; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (aarry[j][i0][i1][i2][i3]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
        k=k;
  }
i1=j1; i2=j2; i3=j3; i4=j4;
for (k=hl+3; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (aarry[i1][i2][i3][i4][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=arr1[k]=b[l];
         i=i;
  }
}

void apass6() {int i0, i1, i2, i3, i4;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  if(aarry[i0][i1][i2][i3][i4]=='Y') lijnk(((i0*KK+i1)*KK+i2)*KK+i3,((i1*KK+i2)*KK+i3)*KK+i4,3);
  }
printf("apass6 done, dd=%3d\n",dd);
}

/* TWO32.C */

/* Pass 1 analyzes all sequences conforming to (2,-2+l)  */
void bpass1(l,p,r) int l, p, r; {
int ix[RR+4], jx[RR], kx;
int i, m, n;
asctobin();
printf("RIJN32 - bpass1: %d, %d, %d\n",l,p,r);
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++)
for (ix[7]=0; ix[7]<KK; ix[7]++)
for (ix[8]=0; ix[8]<KK; ix[8]++) {
  for (i=0; i<RR; i++) 
	jx[i]=binrule[ix[i]][ix[i+1]][ix[i+2]][ix[i+3]][ix[i+4]];
  kx=binrule[jx[0]][jx[1]][jx[2]][jx[3]][jx[4]];
switch (l) {
  case 0:  m=ix[0]; n=kx; break;
  case 1:  m=ix[1]; n=kx; break;
  case 2:  m=ix[2]; n=kx; break;
  case 3:  m=ix[3]; n=kx; break;
  case 4:  m=ix[4]; n=kx; break;
  case 5:  m=ix[5]; n=kx; break;
  case 6:  m=ix[6]; n=kx; break;
  case 7:  m=ix[7]; n=kx; break;
  case 8:  m=ix[8]; n=kx; break;
  case 9:  m=ix[0]; n=jx[2]; break;
  case 10: m=ix[8]; n=jx[2]; break;
  case 11: m=ix[1]; n=jx[2]; break;
  case 12: m=ix[7]; n=jx[2]; break;
  case 13: m=0; n=kx; break;
  case 14: m=1; n=kx; break;
  default: break;}
barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]][ix[7]][ix[8]]=
	(m==n?'Y':'N');
}
if (r) {bpass2i(); bpass2o();}
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void ypass1(l) int l; {
int  i, ix[RR+4], jx[RR], kx;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++)
for (ix[7]=0; ix[7]<KK; ix[7]++)
for (ix[8]=0; ix[8]<KK; ix[8]++)
 {
  for (i=0; i<RR; i++) 
	jx[i]=binrule[ix[i]][ix[i+1]][ix[i+2]][ix[i+3]][ix[i+4]];
  kx=binrule[jx[0]][jx[1]][jx[2]][jx[3]][jx[4]];
  barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]][ix[7]][ix[8]]=
	kx==l?'Y':'N';
  }
bpass2i();
bpass2o();
}

/* Passs 2i flags links which have an incoming arrow */
void bpass2i() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, m;
// printf(" Pass2i\015");
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
if ((barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]&0x5F)=='Y')
 {for (m=0; m<KK; m++) barry[i1][i2][i3][i4][i5][i6][i7][i8][m]|=0x20;}
}
} while (bpass3()!=0); }

/* Passs 2o flags links which have an outgoing arrow */
void bpass2o() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, m;
do {
// printf(" Pass2o\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
if ((barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]&0x5F)=='Y')
 {for (m=0; m<KK; m++) barry[m][i0][i1][i2][i3][i4][i5][i6][i7]|=0x20;}
}
} while (bpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts channges */
int bpass3() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8, l;
// printf(" Pass3 \015");
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
switch (barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]) {
    case 'y': barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]='Y'; break;
    case 'Y': barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]='N'; l=1; break;
    case 'n': case 'N': barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 prints loops which remain */
void bpass4() {
int i0, i1, i2, i3, i4, i5, i6, i7, i8;
int j0, j1, j2, j3, j4, j5, j6, j7, j8;
int k, l, m;
// printf(" pass4 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6; j7=i7; j8=i8; l=0; m=0;
do {
        if (barry[0][j1][j2][j3][j4][j5][j6][j7][j8]=='Y')
    {barry[0][j1][j2][j3][j4][j5][j6][j7][j8]='y';
    k=j8; j8=j7; j7=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (barry[1][j1][j2][j3][j4][j5][j6][j7][j8]=='Y')
    {barry[1][j1][j2][j3][j4][j5][j6][j7][j8]='y';
    k=j8; j8=j7; j7=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=j8; j8=k;} }}
  } while (l==0);
l=0; 
m=0;
do {
        if (barry[j0][j1][j2][j3][j4][j5][j6][j7][0]=='y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,0);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=0; m=1;}
  else {if (barry[j0][j1][j2][j3][j4][j5][j6][j7][1]=='y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,1);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (barry[j0][j1][j2][j3][j4][j5][j6][j7][0]=='Y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,0);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=0; m=1;}
  else {if (barry[j0][j1][j2][j3][j4][j5][j6][j7][1]=='Y')
   {bprf(j0,j1,j2,j3,j4,j5,j6,j7,1);
   barry[j0][j1][j2][j3][j4][j5][j6][j7][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=j7; j7=1; m=1;}
  else {l=1; if (m==1) {kwait(0);} ;}}
  } while (l==0);
}
}

/* print one of the links in a chain */
bprf(i0,i1,i2,i3,i4,i5,i6,i7,i8) int i0, i1, i2, i3, i4, i5, i6, i7, i8; {
int i, j;
kwait(1);
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("%1d",i3);
printf("%1d",i4);
printf("%1d",i5);
printf("%1d",i6);
printf("%1d",i7);
printf("-");
printf("%1d",i8);
printf(" ");
i=((((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7;
j=((((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8;
lijnk(i,j,3);
}

/* Pass 5 - show evolution of sample loops */
void bpass5(c) char c; {
int i0, i1, i2, i3, i4, i5, i6, i7, i8;
int j1, j2, j3, j4, j5, j6, j7, j8;
int hl;
int b[KK], irry[RR+4], i, j, k, l;
printf("bpass5: c = %c\n",c);
  switch (c) {
    case 'z': zerix(irry,RR+4); break;
    case 'r': ranix(irry,RR+4); break;
    case 'n': nodix(irry,RR+3);
	i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3]; 
	i4=irry[4]; i5=irry[5]; i6=irry[6]; i7=irry[7]; 
	for (i=j=0; j<KK; j++) 
	if (barry[i0][i1][i2][i3][i4][i5][i6][i7][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[8]=b[l];
	break;
    case 'i': incix(irry,RR+4); break;
    default:  zerix(irry,RR+4); break;}
for (k=0; k<1024; k++) {
if (barry[irry[0]]
	 [irry[1]]
	 [irry[2]]
	 [irry[3]]
	 [irry[4]]
	 [irry[5]]
	 [irry[6]]
	 [irry[7]]
	 [irry[8]]=='Y') break;
  incix(irry,RR+4);
  }
i0=irry[0]; i1=j1=irry[1]; i2=j2=irry[2]; i3=j3=irry[3]; 
i4=j4=irry[4];
i5=j5=irry[5]; i6=j6=irry[6]; i7=j7=irry[7]; i8=j8=irry[8]; 
hl=AL/2;
arr1[hl-4]=i0;
arr1[hl-3]=i1;
arr1[hl-2]=i2;
arr1[hl-1]=i3;
arr1[hl]=i4;
arr1[hl+1]=i5;
arr1[hl+2]=i6;
arr1[hl+3]=i7;
arr1[hl+4]=i8;
printf("bpass5: initial node =  %d%d%d%d%d%d%d%d%d\n",
	i0,i1,i2,i3,i4,i5,i6,i7,i8);
for (k=hl-5; k>=0; k--) {
  for (i=j=0; j<KK; j++) 
	if (barry[j][i0][i1][i2][i3][i4][i5][i6][i7]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  if (i==1) l=i-1; else l=random()%i;
  i7=i6; i6=i5; i5=i4; i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
printf("bpass5: left half done\n");
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5; i6=j6; i7=j7; i8=j8;
for (k=hl+5; k<AL; k++) {
  for (i=j=0; j<KK; j++) 
	if (barry[i1][i2][i3][i4][i5][i6][i7][i8][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  if (i==1) l=i-1; else l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=i5; i5=i6; i6=i7; i7=i8; i8=arr1[k]=b[l];
  }
printf("bpass5: right half done\n");
}

/* pause at the end of a full screen	    */
/* kwait(0) - short or null line	    */
/* kwait(1) - split long lines as necessary */
/* kwait(2) - wait to continue		    */
/* kwait(3) - reset 			    */
kwait(i) int i; {
switch (i) {
  case 0: printf("\n"); nc=0; nl++; break;
  case 1: if (nc==mc) {printf("&\n"); nc=1; nl++;} else nc++; break;
  case 2: nl=NW; break;
  case 3: nc=0; nl=0; break;
  default: break;}
if (nl==NW) {
  nl=1;
  printf(" press any key to continue\015");
  kbdin();
  printf("                          \015");
  videoscroll(1,0,24,14,0,0);
  videocursor(0,0,0);
  printf("\n");
  }
}

void bpass6() {int i0, i1, i2, i3, i4, i5, i6, i7, i8;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
  if(barry[i0][i1][i2][i3][i4][i5][i6][i7][i8]=='Y')
    lijnk(((((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7,
          ((((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6)*KK+i7)*KK+i8,3);
  }
}

/* end nxrijn32.c */
