
/* SQAR31.C					*/
/* Two-Dimensional Square Lattice Automaton	*/
/* 3-state cells, first neighbors		*/
/* ------- von Neumann neighborhoods -------	*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/
/* 20 July 1988 - von Neumann neighborhgood	*/

#import "nxlcau32.h"

# define GH    (DS/KK)	/* DS/KK= third number of neighborhoods	*/
# define QS	    32  /* number of symmetry classes		*/
# define NF	    50	/* lines in live frequency graph	*/
# define NG	    21	/* number of Golay Classes		*/

char    golRule[KK*NG];	/* evolutionary rule       */
char    sqrule[DS];	/* evolutionary rule       */
char    qarule[DS];	/* auxiliary rule flags    */
int     li, lj;		/* cursor in cell array    */
int	lf[KK];		/* number of live cells	   */
int     gus[2*NG];	/* golay use statistics    */
int     rus[DS];	/* rule use statistics     */
int     tus[TS];	/* totalistic use stats    */
int     gc[DS];		/* Golay code key array    */
int     tc[DS];		/* totalistic key array    */
double  bp[RR+1];	/* Bernstein coefficients  */
double  bbpp[27];	/* 2d Bnstn coefficients   */

extern char   ascrule[KK][KK][KK][KK][KK];
extern int    binrule[KK][KK][KK][KK][KK];

mainsquar() {}	   /* end main */

void asctoqar() {int i0, i1, i2, i3, i4;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
    sqrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*i4)))] = ascrule[i0][i1][i2][i3][i4];
}

void qartoasc() {int i0, i1, i2, i3, i4;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
    ascrule[i0][i1][i2][i3][i4] = sqrule[i0+KK*(i1+KK*(i2+KK*(i3+KK*i4)))];
}

/* calculate one generation of evolution */
/* aa[m][n] = original and final plane   */
/* bb[m][n] = intermediate scratch plane */
/* m = number of rows on the torus       */
/* n = number of columns on the torus    */
aronegen(aa,bb,m,n) char *aa; int *bb, m, n; {int i0, i1, gn;

/* count neighbors of center square */

for (i0=1; i0<m-1; i0++)
for (i1=1; i1<n-1; i1++) {
  gn=aa[n*(i0-1)+i1];
  gn=KK*gn+aa[n*i0+(i1-1)];
  gn=KK*gn+aa[n*(i0+1)+i1];
  gn=KK*gn+aa[n*i0+(i1+1)];
  gn+=GH*aa[n*i0+i1];
  bb[n*i0+i1]=gn;
  }

/* count neighbors for left edge (i1=0) */

for (i0=1; i0<m-1; i0++) {
  gn=aa[n*(i0-1)];
  gn=KK*gn+aa[n*i0-1];
  gn=KK*gn+aa[n*(i0+1)];
  gn=KK*gn+aa[n*i0+1];
  gn+=GH*aa[n*i0];
  bb[n*i0]=gn;
  }

/* count neighbors for right edge (i1=n-1) */

for (i0=1; i0<m-1; i0++) {
  gn=aa[n*(i0-1)+(n-1)];
  gn=KK*gn+aa[n*i0+((n-1)-1)];
  gn=KK*gn+aa[n*(i0+1)+(n-1)];
  gn=KK*gn+aa[n*i0+((n-1)+1)];
  gn+=GH*aa[n*i0+n-1];
  bb[n*i0+(n-1)]=gn;
  }

/* count neighbors for top edge (i0=0) */

for (i1=1; i1<n-1; i1++) {
  gn=aa[n*(m-1)+i1];
  gn=KK*gn+aa[i1-1];
  gn=KK*gn+aa[n+i1];
  gn=KK*gn+aa[i1+1];
  gn+=GH*aa[i1];
  bb[i1]=gn;
  }

/* count neighbors of bottom edge (i0=m-1) */

for (i1=1; i1<n-1; i1++) {
  gn=aa[n*(m-1)+i1];
  gn=KK*gn+aa[n*(m-1)+(i1-1)];
  gn=KK*gn+aa[i1];
  gn=KK*gn+aa[n*(m-1)+(i1+1)];
  gn+=GH*aa[n*(m-1)+i1];
  bb[n*(m-1)+i1]=gn;
  }

/* count neighbors for upper left hand corner */
/* i0 = 0, i1 = 0 			      */

  gn=aa[n*(m-1)];
  gn=KK*gn+aa[n-1];
  gn=KK*gn+aa[n];
  gn=KK*gn+aa[1];
  gn+=GH*aa[0];
  bb[0]=gn;

/* count neighbors for upper right hand corner */
/* i0 = 0, i1 = n-1 			       */

  gn=aa[m*n-1];
  gn=KK*gn+aa[n-1];
  gn=KK*gn+aa[2*n-1];
  gn=KK*gn+aa[0];
  gn+=GH*aa[n-1];
  bb[n-1]=gn;

/* count neighbors for bottom left hand corner */
/* i0 = m-1, i1 = 0 			       */

  gn=aa[(m-1)*n];
  gn=KK*gn+aa[m*n-1];
  gn=KK*gn+aa[0];
  gn=KK*gn+aa[n*(m-1)+1];
  gn+=GH*aa[(m-1)*n];
  bb[(m-1)*n]=gn;

/* count neighbors for bottom right hand corner */
/* i0 = m-1, i1 = n-1 			        */

  gn=aa[(m-1)*(n-1)];
  gn=KK*gn+aa[m*n-1];
  gn=KK*gn+aa[n-1];
  gn=KK*gn+aa[(m-1)*n];
  gn+=GH*aa[m*n-1];
  bb[m*n-1]=gn;

/* phase to update cells */
for (i0=0; i0<KK; i0++)   lf[i0] =0;
for (i0=0; i0<TS; i0++)   tus[i0]=0;
for (i0=0; i0<DS; i0++)   rus[i0]=0;
for (i0=0; i0<2*NG; i0++) gus[i0]=0;
for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) {
  gn=bb[n*i0+i1];
  aa[n*i0+i1]=sqrule[gn]-'0';
  lf[aa[n*i0+i1]]+=1;
  gus[gc[gn]]+=1;
  tus[tc[gn]]+=1;
  rus[gn]++;
  }

/* phase to show results */
}	 /* end aronegen */
    
/*find fraction of live cells in arry[m][n] */
void statecount(aa,m,n) char *aa; int m, n; {int i0, i1;
  for (i0=0; i0<KK; i0++) lf[i0]=0;
  for (i0=0; i0<m; i0++) for (i1=0; i1<n; i1++) lf[aa[n*i0+i1]]+=1;
}

/* display a screen of evolution */
screen(aa,m,n) char *aa; int m, n; {int i, j;
  for (j=0; j<m; j++) for (i=0; i<n; i++) videodot(j,i,aa[n*j+i]);}

/* make a frame for the fixpoint graph  */
/* n     = dimension of arframe         */
argfram(n) int n; {int i;
for (i=0; i<=n; i++) {bdot(i,0,0);bdot(i,n,0);bdot(n,i,0);bdot(0,i,0);};
for (i=0; i<=n; i+=10) {bdot(i,0,3);bdot(i,n,3);bdot(n,i,3);bdot(0,i,3);};
}

/* put a diagonal in the fixpoint graph */
argdiag(n) int n; {int i; for (i=0; 2*i<n; i++) bdot(2*i,2*i,3);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<QS; i++) s+=(double)rus[i];
if  (s<1.0) {for (i=0; i<QS; i++) rus[i]=0; return;};
for (i=0; i<QS; i++) {
  t=333.3*(((double)rus[i])/s);	/* full scale = 1/10 population */
  if (t>=32000.0) t=50.0;	/* larger looks negative */
  n=(int)t;
  sdot(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
  for (j=n; j<m; j++) sdot(i,j,2);
  rus[i]=0;
  }
}

/* graph fraction of live cells */
/* y = column for dot */
/* n = value to be graphed */
arlstat(y,n) int y, n; {int i, k;
  k=(int)(100.0*(((double)(n))/((double)(AL*AL))));
  if (k<=0) k=1; if (k>=100) k=99;
  for (i=0; i<k; i++) ldot(i,y,1);
  for (i=k; i<101; i++) ldot(i,y,2);
  for (i=0; i<101; i++) ldot(i,y+1,0);
  for (i=0; i<101; i+=10) ldot(i,y+1,3);
}

/* generate the totalistic code table */
totta() {int i0, i1, i2, i3, i4;
  for (i4=0; i4<KK; i4++)
  for (i3=0; i3<KK; i3++)
  for (i2=0; i2<KK; i2++)
  for (i1=0; i1<KK; i1++)
  for (i0=0; i0<KK; i0++)
    tc[i0+KK*(i1+KK*(i2+KK*(i3+KK*i4)))]=i0+i1+i2+i3+i4;
}

/* generate the Golay code table */
golta() {
int i, j, k;
int i0, i1, i2, i3;
int binq();
k=0;
for (i3=0; i3<KK; i3++)
for (i2=0; i2<KK; i2++)
for (i1=0; i1<KK; i1++)
for (i0=0; i0<KK; i0++) {
  i=binq(i0,i1,i2,i3);
  j=binq(i1,i2,i3,i0);
  if (j<i) i=j;
  j=binq(i2,i3,i0,i1);
  if (j<i) i=j;
  j=binq(i3,i0,i1,i2);
  if (j<i) i=j;
  j=binq(i0,i3,i2,i1);
  if (j<i) i=j;
  j=binq(i3,i2,i1,i0);
  if (j<i) i=j;
  j=binq(i2,i1,i0,i3);
  if (j<i) i=j;
  j=binq(i1,i0,i3,i2);
  if (j<i) i=j;
  j=binq(i0,i1,i2,i3);
  if (i==j) {gc[i]=k; k++;} else {gc[j]=gc[i];};
  }
printf("no. cl. = %3d\n",k);
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3) int i0, i1, i2, i3; {return i0+KK*(i1+KK*(i2+KK*i3));}

/* calculate Golay class of a cell */
/* no check is made that cell isn't at array edge */
int golcl(i0,i1,arr1) int  i0, i1, arr1[AL][AL]; {int gn;
gn=arr1[i0-1][i1];
gn=KK*gn+arr1[i0][i1-1];
gn=KK*gn+arr1[i0+1][i1];
gn=KK*gn+arr1[i0][i1+1];
if (arr1[i0][i1]==1) gn+=GH;
gn=gc[gn];
return gn;
}

/* plot self-consistent probability graph */
bpanel() {arburnc(); argfram(100); argdiag(100); bargraf(RR,100);}

/* generate Bernstein coefficients */
arburnc() {int i;
  for (i=0; i<RR+1; i++) bp[i]=0.0;
  if (golRule[0]=='1') bp[0]+=1.0;
  if (golRule[1]=='1') bp[1]+=4.0;
  if (golRule[2]=='1') bp[2]+=4.0;
  if (golRule[3]=='1') bp[2]+=2.0;
  if (golRule[4]=='1') bp[3]+=4.0;
  if (golRule[5]=='1') bp[4]+=1.0;
  if (golRule[6]=='1') bp[1]+=1.0;
  if (golRule[7]=='1') bp[2]+=4.0;
  if (golRule[8]=='1') bp[3]+=4.0;
  if (golRule[9]=='1') bp[3]+=2.0;
  if (golRule[10]=='1') bp[4]+=4.0;
  if (golRule[11]=='1') bp[5]+=1.0;
}

/* graph Bernstein polynomial */
bargraf(k,n) int k, n; {int i; double arburn(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
  bdot(i,(int)(en*arburn(p,k)),1);
  }
}

/* evaluate the nth degree Bernstein polynomial at point p */
double arburn(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* calculate the composite square of the rule of evolution */
/*	  	    0		*/
/*	  0	  1 8 7		*/
/*	1 4 3	2 9 c b 6	*/
/*	  2	  3 a 5		*/
/*		    4		*/
/* evaluate composite Bernstein polynomial for component i at point t[KK] */
/* version for planar von Neumann neighborhood 				  */
double pldbern(i,t) int i; double t[KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10, i11, i12;
int    j0, j1, j2, j3, j4;
double a, b, c, d, e, f, g, h, o, p, q, r, s;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++) {
        for (i4=0, e=0.0; i4<KK; i4++) {
          for (i5=0, f=0.0; i5<KK; i5++) {
            for (i6=0, g=0.0; i6<KK; i6++) {
              for (i7=0, h=0.0; i7<KK; i7++) {
                for (i8=0, o=0.0; i8<KK; i8++) {
                  for (i9=0, p=0.0; i9<KK; i9++) {
                    for (i10=0, q=0.0; i10<KK; i10++) {
                      for (i11=0, r=0.0; i11<KK; i11++) {
                        for (i12=0, s=0.0; i12<KK; i12++) {
			   j0=binrule[i0][i1][i12][i7][i8];
			   j1=binrule[i1][i2][i3][i12][i9];
			   j2=binrule[i12][i3][i4][i5][i10];
			   j3=binrule[i7][i12][i5][i6][i11];
			   j4=binrule[i8][i9][i10][i11][i12];
                           if (binrule[j0][j1][j2][j3][j4]==i) s+=t[i12];
	                   }
                        r+=s*t[i11];}
                      q+=r*t[i10];}
                    p+=q*t[i9];}
                  o+=p*t[i8];}
                h+=o*t[i7];}
              g+=h*t[i6];}
            f+=g*t[i5];}
          e+=f*t[i4];}
        d+=e*t[i3];}
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(199-(AL-1-j),60+i,l);}

/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(j,i,l);}

/* ldot(i,j,l) - place dot on live probability  screen at (i,j) in color l */
ldot(i,j,l) int i, j, l; {videodot(90-NF+j,i,l);}

/* sdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
sdot(i,j,l) int i, j, l; {videodot(199-10-j,70+5*i,l);}

/* clear the cell array */
zarry(arr1) int  arr1[AL][AL]; {int i0, i1;
for (i0=0; i0<AL; i0++) for (i1=0; i1<AL; i1++) arr1[i0][i1]=0;
}

/* display rule menu */
armenu() {}

/* display rule number by sum in color */
arscrrul(y,x) int x, y; {int i, n;
n=QS;
videocursor(0,y,x);
  arscrstr("  ");
  for (i=0;   i<n;   i++) resch(i);
videocursor(0,y,x);
}

/* display rule number as a hexadecimal */
harscrrul() {int i, k;
  videocursor(0,0,0);
  printf("    ");
  for (i=0,  k=0; i<12;  i++) k=KK*k+sqrule[i]-'0';
  printf("%3.3X",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
warscrrul() {int i;
  i=sqrule[11]-'0';
  i=KK*i+sqrule[10]-'0';
  i=KK*i+sqrule[3]-'0';
  i=KK*i+sqrule[1] -'0';
  i=KK*i+sqrule[10]-'0';
  i=KK*i+sqrule[9]-'0';
  i=KK*i+sqrule[1] -'0';
  i=KK*i+sqrule[0] -'0';
  videocursor(0,0,32);
  printf("     ");
  videocursor(0,0,32);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* edit the rule */
aredrule() {char c; int i;
  i=0;
  while (0<1) {
//    videocursor(0,TRR,TRC+i+2);
    videoputc(sqrule[i],3);
//    videocursor(0,TRR,TRC+i+2);
    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': 				/* state = 0,1 */
          sqrule[i]=c;
	  resch(i);
          i++; if (i==QS) i=0;
	  break;
        case ' ': case '\315':				/* space = advance */
	  resch(i);
          i++; if (i==QS) i=0;
	  break;
        case '\010': case '\313':			/* backspace */
	  resch(i);
          if (i>0) i--;
	  break;
	case '<':					/* cursor to front */
	  resch(i);
	  i=0;
	  break;
	case '>':					/* cursor to end */
	  resch(i);
	  i=QS-1;
	  break;
	case '\013': case '\310':			/* up arrow */
	  qarule[i]='+';
	  resch(i);
          i++; if (i==QS) i=0;
	  break;
	case '\012': case '\320':			/* down arrow */
	  qarule[i]=' ';
	  resch(i);
          i++; if (i==QS) i=0;
	  break;
	default:
	  break;
          };
    };
}

/* restore character in edrule */
resch(i) int i; {videoputc(sqrule[i],(qarule[i]==' ')?1:2);}

/* edit the plane of cells */
aredline(arr1) int arr1[AL][AL]; {
char c;
int  i, j, k, arr2[AL][AL], golcl();

for (i=0; i<AL; i++) for (j=0; j<AL; j++) arr2[i][j]=0;

videoscroll(19,0,24,25,0,0);
videocursor(0,19,0);
arscrstr("insert states with 0,1,.");
videocursor(0,20,0);
arscrstr("move cursor using arrows");
videocursor(0,21,0);
arscrstr("NSEW R CGO fgmxz *<>^_");
videocursor(0,22,0);
arscrstr("carriage return exits");

while (0<1) {
  adot(li,lj,3);
  videocursor(0,0,1);
  videoputc('?',1);
  c=kbdin();
  videocursor(0,0,1);
  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '.':	/* enter state */
      if (c=='.') c='1';
      i=c-'0';
      arr1[li][lj]=i;
      adot(li,lj,i);
      break;
    case '\012': case '\320':		/* down    */
      adot(li,lj,arr1[li][lj]);
      lj++; if (lj==AL) lj=0;
      break;
    case '\013': case '\310':		/* up      */
      adot(li,lj,arr1[li][lj]);
      if (lj==0) lj=AL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward */
      adot(li,lj,arr1[li][lj]);
      li++; if (li==AL) li=0;
      break;
    case '\010': case '\313':		/* back    */
      adot(li,lj,arr1[li][lj]);
      if (li==0) li=AL; li--;
      break;
    case '<':				/* left margin  */
      adot(li,lj,arr1[li][lj]);
      li=0;
      break;
    case '>':				/* right margin */
      adot(li,lj,arr1[li][lj]);
      li=AL-1;
      break;
    case '^':				/* top margin  */
      adot(li,lj,arr1[li][lj]);
      lj=0;
      break;
   case '_':				/* bottom margin  */
      adot(li,lj,arr1[li][lj]);
      lj=AL-1;
      break;
   case '*':				/* bottom margin  */
      adot(li,lj,arr1[li][lj]);
      li=AL/2; 
      lj=AL/2;
      break;
     case 'f':				/* generate a frame */
//      arframe (2);
      break;
    case 'g':				/* one cycle of evolution */
//      arframe (1);
      aronegen(arr1);
      screen(arr1);
      break;
    case 'm':				/* mark neighborhood */
      qarule[golcl(li,lj,arr1)]='+';
//      arscrrul(TRR,TRC);
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>AL-11) break;
      if (lj<10||lj>AL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        qarule[golcl(i,j,arr1)]='+';
        };};
//      arscrrul(TRR,TRC);
      break;
    case 'x':				/* uncomitted transitions */
      for (k=0; k<QS; k++) qarule[k]=' ';
//      videocursor(0,TRR,TRC);
//      arscrrul(TRR,TRC);
      break;
    case 'z':				/* clear array  */
      zarry(arr1);
      screen(arr1);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<AL; i++) {
        k=arr1[i][0]; 
        for (j=1; j<AL; j++) arr1[i][j-1]=arr1[i][j];
        arr1[i][AL-1]=k;
	};
      if (lj==0) lj=AL; lj--;
      screen(arr1);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<AL; i++) {
        k=arr1[i][AL-1];
        for (j=1; j<AL; j++) arr1[i][AL-j]=arr1[i][AL-j-1];
        arr1[i][0]=k;
	};
      lj++; if (lj==AL) lj=0;
      screen(arr1);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<AL; j++) {
        k=arr1[AL-1][j];
        for (i=1; i<AL; i++) arr1[AL-i][j]=arr1[AL-i-1][j];
        arr1[0][j]=k;
	};
      li++; if (li==AL) li=0;
      screen(arr1);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<AL; j++) {
        k=arr1[0][j];
        for (i=1; i<AL; i++) arr1[i-1][j]=arr1[i][j];
	arr1[AL-1][j]=k;
	};
      if (li==0) li=AL; li--;
      screen(arr1);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<AL; j++) {
      for (i=0; i<j;  i++) {
        k=arr1[i][j];
	arr1[i][j]=arr1[j][i];
        arr1[j][i]=k;
        };};
      k=li; li=lj; lj=k;
      screen(arr1);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr2[i][j]=arr1[i][j];
	};};
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]=arr2[i][j];
	};};
      screen(arr1);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<AL; j++) {
      for (i=0; i<AL; i++) {
	arr1[i][j]|=arr2[i][j];
	};};
      screen(arr1);
      break;
    default:
      break;
      };				/* end switch */
  };					/* end while */
adot(li,lj,arr1[li][lj]);
videoscroll(19,0,24,25,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}				        /* end aredline */

/* generate random cells in arr1 */
ranarr(arr1) int  arr1[AL][AL]; {int i0, i1, i;
for (i=0, i0=0; i0<AL; i0++) for (i1=0; i1<AL; i1++) arr1[i0][i1]=random()%KK;
}

/* generate random cells in the center of arr1 */
rancen(arr1) int  arr1[AL][AL]; {int i0, i1, ia, ib;
  for (i0=0; i0<AL; i0++) 
  for (i1=0; i1<AL; i1++) arr1[i0][i1]=0;
  ia=AL/3; ib=2*(AL/3);
  for (i0=ia; i0<ib; i0++)
  for (i1=ia; i1<ib; i1++) arr1[i0][i1]=random()%KK;
}

/* generate a random rule */
// ranrul() {int i; for (i=0; i<QS; i++) sqrule[i]=random()%KK+'0';}

/* write a string in graphics mode */
arscrstr(s) char *s; {for (; *s != '\0'; s++) videoputc(*s,1); }

# include "nxruin31.c"

/* end nxsqar31.c */
