
/* 
 * SQARView.m -- Implementation file for the SQARView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "SQARView.h"
#import "drawingFuncs.h"
#import "nxlcau32.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 33				/* maximum binomial coefficient */
#define NG 243				/* number of Golay Classes	*/

extern char ascrule[KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char sqrule[];			/* evolutionary rule */

extern int     garry[];
extern char    arr1[], arr2[];
extern double  bp[];			/* Bernstein coefficients */

extern NXColor colst[KK];    		/* Array for state colors */

int    checkBoard=NO;
int    drawARBack=YES;
int    drawBnstein=NO;
int    drawMosaic=NO;
int    drawPlane=NO;
int    lastPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    drawSmall=NO;
int    smallSmall=NO;
int    smallMosaic=NO;

int    repPlEv;

extern int    drawview;
extern int    co, dn;

/* declarations for warnings */
extern nearest();
extern edshf();

@implementation SQARView
 
/* -------------------------------- S Q A R -------------------------------- */

/* put up the principal window for the PairView. Since pair.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "Pair" option in the main menu.	   	  */

- loadSqar:sender {
  if (sqarWindow==nil) {
    if (![NXApp loadNibSection: "sqar.nib" owner: self withNames: NO]) 
	{return nil;}
//    prin=NO;
    }
    dn=2;
    [SQARWindow makeKeyAndOrderFront: sender];
    [sqarWindow makeKeyAndOrderFront: nil];
  return self;
  }

void drawsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void drawcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void manysquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawAgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printSQAR:sender {
    switch (lastPlane) {
      case 0: break;
      case 1: drawPlane=YES;   break;
      case 2: drawMosaic=YES;  break;
      case 3: drawSmall=YES;   break;
      case 4: smallMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lastPlane) {
      case 0: break;
      case 1: drawPlane=NO;   break;
      case 2: drawMosaic=NO;  break;
      case 3: drawSmall=NO;   break;
      case 4: smallMosaic=NO; break;
      default: break;}
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQAR:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQAR:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  *flag = NO;
return self;
}

- copySQAR:sender {
    drawARBack=NO;
    switch (lastPlane) {
      case 0: break;
      case 1: [self copyWhatsit: &drawPlane];   break;
      case 2: [self copyWhatsit: &drawMosaic];  break;
      case 3: [self copyWhatsit: &drawSmall];   break;
      case 4: [self copyWhatsit: &smallMosaic]; break;
      default: break;}
    drawARBack=YES;
return self;
}

- drawPlEvol:sender {int i;
  repPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawPlane=YES;
  asctobin();
  if (repPlEv==0) [self display]; else for (i=0; i<repPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: repPlEv];
  drawPlane=NO;
  lastPlane=1;
return self;
}

- printPlEvol:sender {
    repPlEv=0;
    drawview=7;
    drawPlane=YES;
    [keyWindow smartPrintPSCode: self];
    drawPlane=NO;

return self;
}

- drawMosaic:sender {
  drawview=11;
  drawMosaic=YES;
  [self display];
  drawMosaic=NO;
  lastPlane=2;
return self;
}

- printMosaic:sender {
  drawview=11;
  drawMosaic=YES;
  [keyWindow smartPrintPSCode: self];
  drawMosaic=NO;
return self;
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

- resetNHood:sender {int i, j;
  for (i=0; i<3; i++) for (j=0; j<3; j++) 
	[[nHood cellAt: i: j] setIntValue: NO];
  [nHoodValue setIntValue: ascrule[0][0][0][0][0]-'0'];
return self;
}

- sayRule:sender {
  [nHoodValue setIntValue: ascrule[[[nHood cellAt: 0: 1] state]]
				  [[[nHood cellAt: 1: 0] state]]	
				  [[[nHood cellAt: 1: 2] state]]
				  [[[nHood cellAt: 2: 1] state]]
				  [[[nHood cellAt: 1: 1] state]]-'0'];
return self;
}

- readRule:sender {int which, value;
  which = [[nHood cellAt: 0: 1] state] + KK*(
	  [[nHood cellAt: 1: 0] state] + KK*(	
	  [[nHood cellAt: 1: 2] state] + KK*(
	  [[nHood cellAt: 2: 1] state] + KK*(
	  [[nHood cellAt: 1: 1] state]))));
  switch ([sender selectedRow]) {
    case 0:  if (sqrule[which] = '0') {sqrule[which] = '1'; value = 1;}
	   		         else {sqrule[which] = '0'; value = 0;}
	     break;
    case 1:  value = [nHoodValue intValue]; break;
    case 2:  sqrule[which] = '0'; value = 0; break;
    case 3:  sqrule[which] = '1'; value = 1; break;
    default: value = [nHoodValue intValue]; break;}
  [nHoodValue setIntValue: value];
  qartoasc();
  [rulePanel setAllRules: nil];
  [self copyRule: nil];
  switch ([mMent selectedRow]) {
    case 0: which--; if (which<0) which=DS-1; break;
    case 1: break;
    case 2: which++; if (which>DS-1) which=0; break;
    default: break;}
  [[nHood cellAt: 0: 1] setIntValue: which%KK]; which/=KK;
  [[nHood cellAt: 1: 0] setIntValue: which%KK]; which/=KK;	
  [[nHood cellAt: 1: 2] setIntValue: which%KK]; which/=KK;
  [[nHood cellAt: 2: 1] setIntValue: which%KK]; which/=KK;
  [[nHood cellAt: 1: 1] setIntValue: which%KK]; which/=KK;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1],p[2]);
return self;
}

- sCellLine:sender {
  smallSmall=YES;
  [planeView display];
  smallSmall=NO;
return self;
}

- checkerBoard:sender {
  checkBoard=YES;
  [planeView display];
  checkBoard=NO;
return self;
}

- uniformLine:sender {ranblin(0.333,0.333,0.333); return self;}

- uprule:sender {suggrule(0); [rulePanel setAllRules: self]; return self;}
- lerule:sender {suggrule(1); [rulePanel setAllRules: self]; return self;}
- dorule:sender {suggrule(3); [rulePanel setAllRules: self]; return self;}
- rirule:sender {suggrule(2); [rulePanel setAllRules: self]; return self;}
- idrule:sender {suggrule(4); [rulePanel setAllRules: self]; return self;}
- mjrule:sender {suggrule(5); [rulePanel setAllRules: self]; return self;}
- earule:sender {suggrule(6); [rulePanel setAllRules: self]; return self;}
- serule:sender {suggrule(7); [rulePanel setAllRules: self]; return self;}

void suggrule(i) int i; {int j, k[KK], i0, i1, i2, i3, i4;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: for (j=0; j<KK; j++) k[j]=0;
	      k[i0]++; k[i1]++; k[i2]++; k[i3]++; k[i4]++;
	      if      (k[0]>2) j=0;
	      else if (k[1]>2) j=1;
	      else if (k[2]>2) j=2; 
	      else if (k[0]==1) j=0; 
	      else if (k[1]==1) j=1; 
	      else if (k[2]==1) j=2;
	         else           j=i4; 
	      break;
      case 6: j=(i4+1)%KK;
	      if (j==i0 || j==i1 || j==i2 || j==i3) break;
	      else j=i4; break;
      case 7: j=(i4+1)%KK;
	      if (j==i0 || j==i1 || j==i2 || j==i3) break;
      	      j=(KK+i4-1)%KK;
	      if (j==i0 || j==i1 || j==i2 || j==i3) break;
	      else j=i4; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {
    drawview=7;
    drawSmall=YES;
    repPlEv=[[sender selectedCell] tag];
    if ([reStore state]) savearry();
    [self display];
    if ([reStore state]) renewarry();
    drawSmall=NO;
    lastPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawSmall=YES;
    repPlEv=0;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawSmall=NO;
return self;
}

- drawSmMosaic:sender {
    drawview=11;
    smallMosaic=YES;
    savearry();
    [self display];
    renewarry();
    smallMosaic=NO;
    lastPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallMosaic=NO;
return self;
}

/* requires suitably large character arrays arr1 and arr2 */
/* m = number of rows in small plane (torus)              */
/* n = number of columns in small plane (torus)           */
void smallevolve(m,n) int m, n; {int i, j;
  asctoqar();
  for (j=0; j<m; j++) for (i=0; i<n; i++) videodot(j,i,arr1[n*j+i]);
  aronegen(arr1,garry,m,n);
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (m,n) */
void manyrect(i,j,u,v,m,n) int i, j, u, v, m, n; {int k, l;
  asctoqar();
  for (k=0; k<m; k++) for (l=0; l<n; l++) videodot(i*m+k,j*n+l,arr1[k*n+l]+1);
  aronegen(arr1,garry,m,n);
}

/* m = high/4, n = wide/4 */
void smallgrid(m,n) int m, n; {int i; double em, en;
  em=(double)m;
  en=(double)n;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(en*i),0.0); PSlineto((double)(en*i),4.0*em);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(em*i)); PSlineto(4.0*en,(double)(em*i));}
  PSstroke();
}

/* - - - - - - - - - - -  G O L A Y   S U R R O U N D S  - - - - - - - - - */

- resetGolay:sender {int i, j;
  for (i=0; i<NG; i++) for (j=0; j<KK; j++) 
	[[golMatrix cellAt: i: j] setIntValue: NO];
return self;
}

- copyGolRule:sender {int i, j;
  for (i=0; i<NG; i++) for (j=0; j<KK; j++) 
	[[golMatrix cellAt: i: j] setIntValue: golRule[i+NG*j]];
return self;
}

- enterGolRule:sender {int i, j; 
  for (i=0; i<NG; i++) for (j=0; j<KK; j++) 
	golRule[i+NG*j] = [[golMatrix cellAt: i: j] intValue];
return self;
}

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    drawBnstein=YES;
    [self display];
    drawBnstein=NO;
    [window setDocEdited:YES];
return self;
}

- printBern:sender {
   drawview=12;
   drawBnstein=YES;
   [keyWindow printPSCode:self];
   drawBnstein=NO;
return self;
}

- enterSemiBern:sender {
      asczero(ascrule);
	if ([[bernZMatrix cellAt: 5: 0] state]) ascrule[0][0][0][0][0]='1';
	if ([[bernZMatrix cellAt: 2: 1] state]) ascrule[0][0][0][1][0]='1';
	if ([[bernZMatrix cellAt: 3: 1] state]) ascrule[0][0][1][0][0]='1';
	if ([[bernZMatrix cellAt: 4: 1] state]) ascrule[0][1][0][0][0]='1';
	if ([[bernZMatrix cellAt: 5: 1] state]) ascrule[1][0][0][0][0]='1';
	if ([[bernZMatrix cellAt: 0: 2] state]) ascrule[0][0][1][1][0]='1';
	if ([[bernZMatrix cellAt: 1: 2] state]) ascrule[0][1][1][0][0]='1';
	if ([[bernZMatrix cellAt: 2: 2] state]) ascrule[0][1][0][1][0]='1';
	if ([[bernZMatrix cellAt: 3: 2] state]) ascrule[1][0][0][1][0]='1';
	if ([[bernZMatrix cellAt: 4: 2] state]) ascrule[1][0][1][0][0]='1';
	if ([[bernZMatrix cellAt: 5: 2] state]) ascrule[1][1][0][0][0]='1';
	if ([[bernZMatrix cellAt: 2: 3] state]) ascrule[1][1][1][0][0]='1';
	if ([[bernZMatrix cellAt: 3: 3] state]) ascrule[1][1][0][1][0]='1';
	if ([[bernZMatrix cellAt: 4: 3] state]) ascrule[1][0][1][1][0]='1';
	if ([[bernZMatrix cellAt: 5: 3] state]) ascrule[0][1][1][1][0]='1';
	if ([[bernZMatrix cellAt: 5: 4] state]) ascrule[1][1][1][1][0]='1';
	if ([[bernOMatrix cellAt: 5: 0] state]) ascrule[0][0][0][0][1]='1';
	if ([[bernOMatrix cellAt: 2: 1] state]) ascrule[0][0][0][1][1]='1';
	if ([[bernOMatrix cellAt: 3: 1] state]) ascrule[0][0][1][0][1]='1';
	if ([[bernOMatrix cellAt: 4: 1] state]) ascrule[0][1][0][0][1]='1';
	if ([[bernOMatrix cellAt: 5: 1] state]) ascrule[1][0][0][0][1]='1';
	if ([[bernOMatrix cellAt: 0: 2] state]) ascrule[0][0][1][1][1]='1';
	if ([[bernOMatrix cellAt: 1: 2] state]) ascrule[0][1][1][0][1]='1';
	if ([[bernOMatrix cellAt: 2: 2] state]) ascrule[0][1][0][1][1]='1';
	if ([[bernOMatrix cellAt: 3: 2] state]) ascrule[1][0][0][1][1]='1';
	if ([[bernOMatrix cellAt: 4: 2] state]) ascrule[1][0][1][0][1]='1';
	if ([[bernOMatrix cellAt: 5: 2] state]) ascrule[1][1][0][0][1]='1';
	if ([[bernOMatrix cellAt: 2: 3] state]) ascrule[1][1][1][0][1]='1';
	if ([[bernOMatrix cellAt: 3: 3] state]) ascrule[1][1][0][1][1]='1';
	if ([[bernOMatrix cellAt: 4: 3] state]) ascrule[1][0][1][1][1]='1';
	if ([[bernOMatrix cellAt: 5: 3] state]) ascrule[0][1][1][1][1]='1';
	if ([[bernOMatrix cellAt: 5: 4] state]) ascrule[1][1][1][1][1]='1';
     [rulePanel setAllRules: self];
     [planeWindow setDocEdited: NO];
return self;
}

- resetBern:sender {int i, j; 
    for (i=0; i<TS-1; i++) for (j=0; j<MB; j++) {
	[[bernOMatrix cellAt: j: i] setIntValue: NO];
	[[bernZMatrix cellAt: j: i] setIntValue: NO];
	}
}

- copyRule:sender {
	[[bernZMatrix cellAt: 5: 0] setIntValue: ascrule[0][0][0][0][0]-'0'];
	[[bernZMatrix cellAt: 2: 1] setIntValue: ascrule[0][0][0][1][0]-'0'];
	[[bernZMatrix cellAt: 3: 1] setIntValue: ascrule[0][0][1][0][0]-'0'];
	[[bernZMatrix cellAt: 4: 1] setIntValue: ascrule[0][1][0][0][0]-'0'];
	[[bernZMatrix cellAt: 5: 1] setIntValue: ascrule[1][0][0][0][0]-'0'];
	[[bernZMatrix cellAt: 0: 2] setIntValue: ascrule[0][0][1][1][0]-'0'];
	[[bernZMatrix cellAt: 1: 2] setIntValue: ascrule[0][1][1][0][0]-'0'];
	[[bernZMatrix cellAt: 2: 2] setIntValue: ascrule[0][1][0][1][0]-'0'];
	[[bernZMatrix cellAt: 3: 2] setIntValue: ascrule[1][0][0][1][0]-'0'];
	[[bernZMatrix cellAt: 4: 2] setIntValue: ascrule[1][0][1][0][0]-'0'];
	[[bernZMatrix cellAt: 5: 2] setIntValue: ascrule[1][1][0][0][0]-'0'];
	[[bernZMatrix cellAt: 2: 3] setIntValue: ascrule[1][1][1][0][0]-'0'];
	[[bernZMatrix cellAt: 3: 3] setIntValue: ascrule[1][1][0][1][0]-'0'];
	[[bernZMatrix cellAt: 4: 3] setIntValue: ascrule[1][0][1][1][0]-'0'];
	[[bernZMatrix cellAt: 5: 3] setIntValue: ascrule[0][1][1][1][0]-'0'];
	[[bernZMatrix cellAt: 5: 4] setIntValue: ascrule[1][1][1][1][0]-'0'];
	[[bernOMatrix cellAt: 5: 0] setIntValue: ascrule[0][0][0][0][1]-'0'];
	[[bernOMatrix cellAt: 2: 1] setIntValue: ascrule[0][0][0][1][1]-'0'];
	[[bernOMatrix cellAt: 3: 1] setIntValue: ascrule[0][0][1][0][1]-'0'];
	[[bernOMatrix cellAt: 4: 1] setIntValue: ascrule[0][1][0][0][1]-'0'];
	[[bernOMatrix cellAt: 5: 1] setIntValue: ascrule[1][0][0][0][1]-'0'];
	[[bernOMatrix cellAt: 0: 2] setIntValue: ascrule[0][0][1][1][1]-'0'];
	[[bernOMatrix cellAt: 1: 2] setIntValue: ascrule[0][1][1][0][1]-'0'];
	[[bernOMatrix cellAt: 2: 2] setIntValue: ascrule[0][1][0][1][1]-'0'];
	[[bernOMatrix cellAt: 3: 2] setIntValue: ascrule[1][0][0][1][1]-'0'];
	[[bernOMatrix cellAt: 4: 2] setIntValue: ascrule[1][0][1][0][1]-'0'];
	[[bernOMatrix cellAt: 5: 2] setIntValue: ascrule[1][1][0][0][1]-'0'];
	[[bernOMatrix cellAt: 2: 3] setIntValue: ascrule[1][1][1][0][1]-'0'];
	[[bernOMatrix cellAt: 3: 3] setIntValue: ascrule[1][1][0][1][1]-'0'];
	[[bernOMatrix cellAt: 4: 3] setIntValue: ascrule[1][0][1][1][1]-'0'];
	[[bernOMatrix cellAt: 5: 3] setIntValue: ascrule[0][1][1][1][1]-'0'];
	[[bernOMatrix cellAt: 5: 4] setIntValue: ascrule[1][1][1][1][1]-'0'];
     [planeWindow setDocEdited: NO];
return self;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new SQARView object.  First, an initFrame: message is sent 
 * to super to initialize SQARView as a View.  Next, the SQARView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the SQARView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the SQARView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although SQARView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
        char             ch[WNML], c[80];
	int		 i, j, k, l, n;
        double           p, q, r, s, t, dp, pee, cue;
	NXPoint		 *aPoint;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawARBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

   wrru(ch,ascrule);
   [ruleNumber setStringValue: ch];

	if (drawBnstein==YES) {
	  for (i=0; i<TS; i++) bp[i]=0.0;
	  for (i=0; i<TS-1; i++) for (j=0; j<MB; j++) {
		if ([[bernZMatrix cellAt: j: i] state]) bp[i]+=1.0;
		if ([[bernOMatrix cellAt: j: i] state]) bp[i+1]+=1.0;
		}
	  argfram(50,3);
	  argdiag(50,3);
	  bargraf(TS-1,50);
	  }

        if (smallSmall==YES) {
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);
	  i=(int)(plotRect.size.height/4.0);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (checkBoard==YES) {
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);
	  i=(int)(plotRect.size.height/4.0);
	  zerline();
        for (k=0; k<(i/3); k++) {
	   for (l=0; l<(j/3); l++)     arr1[k*j+l]=0;
           for (l=j/3; l<2*(j/3); l++) arr1[k*j+l]=1;
           for (l=2*(j/3); l<j; l++)  arr1[k*j+l]=2;
           }
        for (k=i/3; k<2*(i/3); k++) {
	   for (l=0; l<(j/3); l++)     arr1[k*j+l]=1;
           for (l=j/3; l<2*(j/3); l++) arr1[k*j+l]=2;
           for (l=2*(j/3); l<j; l++)  arr1[k*j+l]=0;
           }
        for (k=2*(i/3); k<i; k++) {
	   for (l=0; l<(j/3); l++)     arr1[k*j+l]=2;
           for (l=j/3; l<2*(j/3); l++) arr1[k*j+l]=0;
           for (l=2*(j/3); l<j; l++)  arr1[k*j+l]=1;
           }
	  }

        if (drawSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);
	  i=(int)(plotRect.size.height/4.0);
	  for (k=0; k<repPlEv+1; k++) {smallevolve(i,j); [window flushWindow];}
	  }

        if (smallMosaic==YES) {
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/4.0);
	  i=(int)(plotRect.size.height/4.0);
	  for (k=0; k<4; k++) for (l=0; l<4; l++) {
	    manyrect(k,l,4,4,i,j);
	    [window flushWindow];
	    }
	  smallgrid(i,j);
	  }

        if (drawPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) Plane Evolution"];
	drawsquare();
	break;
     case 3: 
	[window setTitle: "3-D Solid Tetrahedron Evolution"];
	drawcube();
	drawgrid();
	break;
     default: [window setTitle: "SQAR22 Plane Evolution"];break;}
	}

        if (drawMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manysquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawgrid();
	break;
     default: [window setTitle: "SQAR22 Plane Evolution"];break;}
	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the SQARView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview==10||drawview==11) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the SQARView.  The SQARView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the SQARView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the SQARView, the SQARView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end SQARView.m */
